from dataclasses import dataclass
from enum import IntEnum

from terrex.net.streamer import Reader, Writer


class BestiaryUnlockType(IntEnum):
    Kill = 0
    Sight = 1
    Chat = 2


@dataclass
class Bestiary:
    unlock_type: BestiaryUnlockType
    npc_net_id: int
    kill_count: int | None = None

    @classmethod
    def read(cls, reader: Reader) -> 'Bestiary':
        unlock_type = BestiaryUnlockType(reader.read_byte())
        npc_net_id = reader.read_short()
        kill_count = (
            reader.read_7bit_encoded_int() if unlock_type == BestiaryUnlockType.Kill else None
        )
        return cls(unlock_type, npc_net_id, kill_count)

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.unlock_type.value)
        writer.write_short(self.npc_net_id)
        if self.unlock_type == BestiaryUnlockType.Kill:
            assert self.kill_count is not None
            writer.write_7bit_encoded_int(self.kill_count)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.rgb import Rgb
from terrex.packet.base import SyncPacket


class SyncPlayer(SyncPacket):
    id = MessageID.SyncPlayer

    def __init__(
        self,
        player_id: int = 0,
        skin_variant: int = 0,
        voice_variant: int = 0,
        voice_pitch_offset: float = 0.0,
        hair: int = 0,
        name: str = "",
        hair_dye: int = 255,
        accessory_visibility: int = 0,
        hide_misc: bool = False,
        hair_color: Rgb | None = None,
        skin_color: Rgb | None = None,
        eye_color: Rgb | None = None,
        shirt_color: Rgb | None = None,
        under_shirt_color: Rgb | None = None,
        pants_color: Rgb | None = None,
        shoe_color: Rgb | None = None,
        difficulty_flags: int = 0,
        biome_torch_flags: int = 0,
        consumables_flags: int = 0,
    ):
        self.player_id = player_id
        self.skin_variant = skin_variant
        self.voice_variant = voice_variant
        self.voice_pitch_offset = voice_pitch_offset
        self.hair = hair
        self.name = name
        self.hair_dye = hair_dye
        self.accessory_visibility = accessory_visibility
        self.hide_misc = hide_misc
        self.hair_color = hair_color or Rgb(255, 255, 255)
        self.skin_color = skin_color or Rgb(255, 255, 255)
        self.eye_color = eye_color or Rgb(255, 255, 255)
        self.shirt_color = shirt_color or Rgb(255, 255, 255)
        self.under_shirt_color = under_shirt_color or Rgb(255, 255, 255)
        self.pants_color = pants_color or Rgb(255, 255, 255)
        self.shoe_color = shoe_color or Rgb(255, 255, 255)
        self.difficulty_flags = difficulty_flags
        self.biome_torch_flags = biome_torch_flags
        self.consumables_flags = consumables_flags

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.skin_variant = reader.read_byte()
        self.voice_variant = reader.read_byte()
        self.voice_pitch_offset = reader.read_float()
        self.hair = reader.read_byte()
        self.name = reader.read_dotnet_string()
        self.hair_dye = reader.read_byte()
        self.accessory_visibility = reader.read_ushort()
        self.hide_misc = reader.read_bool()
        self.hair_color = Rgb.read(reader)
        self.skin_color = Rgb.read(reader)
        self.eye_color = Rgb.read(reader)
        self.shirt_color = Rgb.read(reader)
        self.under_shirt_color = Rgb.read(reader)
        self.pants_color = Rgb.read(reader)
        self.shoe_color = Rgb.read(reader)
        self.difficulty_flags = reader.read_byte()
        self.biome_torch_flags = reader.read_byte()
        self.consumables_flags = reader.read_byte()

    def write(self, writer: Writer):
        writer.write_byte(self.player_id)
        writer.write_byte(self.skin_variant)
        writer.write_byte(self.voice_variant)
        writer.write_float(self.voice_pitch_offset)
        writer.write_byte(self.hair)
        writer.write_dotnet_string(self.name)
        writer.write_byte(self.hair_dye)
        writer.write_ushort(self.accessory_visibility)
        writer.write_bool(self.hide_misc)
        self.hair_color.write(writer)
        self.skin_color.write(writer)
        self.eye_color.write(writer)
        self.shirt_color.write(writer)
        self.under_shirt_color.write(writer)
        self.pants_color.write(writer)
        self.shoe_color.write(writer)
        writer.write_byte(self.difficulty_flags)
        writer.write_byte(self.biome_torch_flags)
        writer.write_byte(self.consumables_flags)

    def get_hide_visible_accessory(self, index: int) -> bool:
        """Get if accessory at index is hidden (bit index in accessory_visibility)."""
        return bool((self.accessory_visibility & (1 << index)) != 0)

    def set_hide_visible_accessory(self, index: int, hide: bool):
        """Set if accessory at index is hidden."""
        mask = 1 << index
        if hide:
            self.accessory_visibility |= mask
        else:
            self.accessory_visibility &= ~mask

    def set_difficulty(self, difficulty: int):
        """Set player difficulty: 0-journey, 1-classic, 2-mediumcore, 3-hardcore."""
        self.difficulty_flags &= ~(1 | (1 << 1) | (1 << 3))  # clear difficulty bits
        if difficulty == 1:
            self.difficulty_flags |= 1 << 0
        elif difficulty == 2:
            self.difficulty_flags |= 1 << 1
        elif difficulty == 3:
            self.difficulty_flags |= 1 << 3

    def get_difficulty(self) -> int:
        """Get player difficulty."""
        if self.difficulty_flags & 1:
            return 1
        if self.difficulty_flags & (1 << 1):
            return 2
        if self.difficulty_flags & (1 << 3):
            return 3
        return 0

    @property
    def extra_accessory(self) -> bool:
        return bool(self.difficulty_flags & (1 << 2))

    @extra_accessory.setter
    def extra_accessory(self, value: bool):
        mask = 1 << 2
        if value:
            self.difficulty_flags |= mask
        else:
            self.difficulty_flags &= ~mask

    @property
    def using_biome_torches(self) -> bool:
        return bool(self.biome_torch_flags & 1)

    @using_biome_torches.setter
    def using_biome_torches(self, value: bool):
        mask = 1
        if value:
            self.biome_torch_flags |= mask
        else:
            self.biome_torch_flags &= ~mask

    @property
    def happy_fun_torch_time(self) -> bool:
        return bool(self.biome_torch_flags & (1 << 1))

    @happy_fun_torch_time.setter
    def happy_fun_torch_time(self, value: bool):
        mask = 1 << 1
        if value:
            self.biome_torch_flags |= mask
        else:
            self.biome_torch_flags &= ~mask

    @property
    def unlocked_biome_torches(self) -> bool:
        return bool(self.biome_torch_flags & (1 << 2))

    @unlocked_biome_torches.setter
    def unlocked_biome_torches(self, value: bool):
        mask = 1 << 2
        if value:
            self.biome_torch_flags |= mask
        else:
            self.biome_torch_flags &= ~mask

    @property
    def unlocked_super_cart(self) -> bool:
        return bool(self.biome_torch_flags & (1 << 3))

    @unlocked_super_cart.setter
    def unlocked_super_cart(self, value: bool):
        mask = 1 << 3
        if value:
            self.biome_torch_flags |= mask
        else:
            self.biome_torch_flags &= ~mask

    @property
    def enabled_super_cart(self) -> bool:
        return bool(self.biome_torch_flags & (1 << 4))

    @enabled_super_cart.setter
    def enabled_super_cart(self, value: bool):
        mask = 1 << 4
        if value:
            self.biome_torch_flags |= mask
        else:
            self.biome_torch_flags &= ~mask

    @property
    def used_aegis_crystal(self) -> bool:
        return bool(self.consumables_flags & 1)

    @used_aegis_crystal.setter
    def used_aegis_crystal(self, value: bool):
        mask = 1
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def used_aegis_fruit(self) -> bool:
        return bool(self.consumables_flags & (1 << 1))

    @used_aegis_fruit.setter
    def used_aegis_fruit(self, value: bool):
        mask = 1 << 1
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def used_arcane_crystal(self) -> bool:
        return bool(self.consumables_flags & (1 << 2))

    @used_arcane_crystal.setter
    def used_arcane_crystal(self, value: bool):
        mask = 1 << 2
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def used_galaxy_pearl(self) -> bool:
        return bool(self.consumables_flags & (1 << 3))

    @used_galaxy_pearl.setter
    def used_galaxy_pearl(self, value: bool):
        mask = 1 << 3
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def used_gummy_worm(self) -> bool:
        return bool(self.consumables_flags & (1 << 4))

    @used_gummy_worm.setter
    def used_gummy_worm(self, value: bool):
        mask = 1 << 4
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def used_ambrosia(self) -> bool:
        return bool(self.consumables_flags & (1 << 5))

    @used_ambrosia.setter
    def used_ambrosia(self, value: bool):
        mask = 1 << 5
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask

    @property
    def ate_artisan_bread(self) -> bool:
        return bool(self.consumables_flags & (1 << 6))

    @ate_artisan_bread.setter
    def ate_artisan_bread(self, value: bool):
        mask = 1 << 6
        if value:
            self.consumables_flags |= mask
        else:
            self.consumables_flags &= ~mask



from .net_ambience_module import NetAmbienceModule
from .net_banners_module import BannersMessageType, NetBannersModule
from .net_bestiary_module import NetBestiaryModule
from .net_crafting_request_module import NetCraftingRequestModule
from .net_creative_power_permissions_module import NetCreativePowerPermissionsModule
from .net_creative_powers_module import NetCreativePowersModule
from .net_creative_unlocks_player_report_module import (
    NetCreativeUnlocksPlayerReportModule,
)

# from .net_crafting_response_module import NetCraftingResponseModule
from .net_leashed_entity_module import LeashedEntityMessageType, NetLeashedEntityModule
from .net_liquid_module import NetLiquidModule
from .net_module import net_module_registry
from .net_particles_module import NetParticlesModule
from .net_ping_module import NetPingModule
from .net_tag_effect_module import NetTagEffectModule, TagEffectMessageType
from .net_teleport_pylon_module import NetTeleportPylonModule, TeleportPylonOperation
from .net_text_module import NetTextModule
from .net_unbreakable_wall_scan_module import NetUnbreakableWallScanModule

__all__ = [
    "NetAmbienceModule",
    "BannersMessageType",
    "NetBannersModule",
    "NetBestiaryModule",
    "NetCraftingRequestModule",
    "NetCreativePowerPermissionsModule",
    "NetCreativePowersModule",
    "NetCreativeUnlocksPlayerReportModule",
    "LeashedEntityMessageType",
    "NetLeashedEntityModule",
    "NetLiquidModule",
    "net_module_registry",
    "NetParticlesModule",
    "NetPingModule",
    "NetTagEffectModule",
    "TagEffectMessageType",
    "NetTeleportPylonModule",
    "TeleportPylonOperation",
    "NetTextModule",
    "NetUnbreakableWallScanModule",
]



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import SyncPacket

# Константы для флагов PulleyMode
PULLEY_HAS_VEL = 0x04

# Константы для флагов PlayerAction
PLAYER_ACTION_HAS_ORIG_AND_HOME_POS = 0x40


class PlayerControls(SyncPacket):
    id = MessageID.PlayerControls

    def __init__(
        self,
        player_id: int = 0,
        keys: int = 0,
        pulley: int = 0,
        action: int = 0,
        sleep_info: int = 0,
        selected_item: int = 0,
        pos: Vec2 | None = None,
        vel: Vec2 | None = None,
        original_and_home_pos: tuple[Vec2, Vec2] | None = None,
    ):
        self.player_id = player_id
        self.keys = keys
        self.pulley = pulley
        self.action = action
        self.sleep_info = sleep_info
        self.selected_item = selected_item
        self.world_position = pos or Vec2(0.0, 0.0)
        self.velocity = vel
        self.original_and_home_pos = original_and_home_pos

    def read(self, reader: Reader) -> None:
        self.player_id = reader.read_byte()
        self.keys = reader.read_byte()
        self.pulley = reader.read_byte()
        self.action = reader.read_byte()
        self.sleep_info = reader.read_byte()
        self.selected_item = reader.read_byte()
        self.world_position = Vec2.read(reader)

        self.velocity = None
        if self.pulley & PULLEY_HAS_VEL:
            self.velocity = Vec2.read(reader)

        self.original_and_home_pos = None
        if self.action & PLAYER_ACTION_HAS_ORIG_AND_HOME_POS:
            orig_pos = Vec2.read(reader)
            home_pos = Vec2.read(reader)
            self.original_and_home_pos = (orig_pos, home_pos)

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.player_id)
        writer.write_byte(self.keys)

        pulley_flags = self.pulley
        if self.velocity is not None:
            pulley_flags |= PULLEY_HAS_VEL
        writer.write_byte(pulley_flags)

        action_flags = self.action
        if self.original_and_home_pos is not None:
            action_flags |= PLAYER_ACTION_HAS_ORIG_AND_HOME_POS
        writer.write_byte(action_flags)

        writer.write_byte(self.sleep_info)
        writer.write_byte(self.selected_item)
        self.world_position.write(writer)

        if self.velocity is not None:
            self.velocity.write(writer)

        if self.original_and_home_pos is not None:
            self.original_and_home_pos[0].write(writer)
            self.original_and_home_pos[1].write(writer)



from terrex.event.types import ChatEvent
from terrex.id import MessageID
from terrex.net.module import (
    NetTextModule,
    net_module_registry,
)
from terrex.net.module.net_module import NetModule
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class NetModules(SyncPacket):
    id = MessageID.NetModules
    module: NetModule | None = None

    def __init__(self, module: NetModule | None = None):
        self.module = module

    def read(self, reader: Reader) -> None:
        module_id = reader.read_ushort()
        module_cls = net_module_registry.get(module_id)
        if module_cls is None:
            raise ValueError(f"Unknown NetModule variant: {module_id}")
        if not issubclass(module_cls, NetModule):
            raise ValueError(f"Registry entry for {module_id} is not a subclass of NetModule")
        module = module_cls()
        module.read(reader)
        self.module = module

    def write(self, writer: Writer) -> None:
        if self.module is None:
            raise ValueError("NetModules module is None during write")
        writer.write_ushort(self.module.id)
        self.module.write(writer)

    async def handle(self, world, player, evman):
        if (
            isinstance(self.module, NetTextModule)
            and self.module.author_id is not None
            and self.module.net_text is not None
            # ignore client chat commands
            and self.module.chat_command_id is None
        ):
            evman.raise_event(
                ChatEvent(
                    self,
                    self.module.author_id,
                    self.module.net_text.text,
                    self.module.chat_command_id,
                )
            )



"""
BiomeConversionID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class BiomeConversionID(IntEnum):
    Purity = 0
    Corruption = 1
    Hallow = 2
    GlowingMushroom = 3
    Crimson = 4
    Sand = 5
    Snow = 6
    Dirt = 7
    ChlorophyteSpread = 8
    ChlorophyteDefenseWhenChloroSpreads = 9
    ChlorophyteDefenseWhenEvilSpreads = 10
    PurificationPowder = 11
    ForcedCorruption = 12
    ForcedCrimson = 13
    ForcedHallow = 14
    ForcedGlowingMushroom = 15
    ForcedSand = 16
    ForcedSnow = 17
    ForcedDirt = 18
    ForcedJungle = 19
    ForcedTemple = 20
    ForcedDungeon = 21




from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ServerPacket


class DevCommands(ServerPacket):
    id = MessageID.DevCommands

    def __init__(self, buf: bytes = b""):
        self.buf = buf

    def read(self, reader: Reader):
        self.buf = reader.remaining()

    def write(self, writer: Writer):
        writer.write_bytes(self.buf)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class SyncCavernMonsterType(SyncPacket):
    id = MessageID.SyncCavernMonsterType

    def __init__(self, types: list[int] | None = None):
        self.types = types or [0] * 6

    def read(self, reader: Reader) -> None:
        self.types = [reader.read_ushort() for _ in range(6)]

    def write(self, writer: Writer) -> None:
        for t in self.types:
            writer.write_ushort(t)



from terrex.id import MessageID
from terrex.packet.base import ServerPacket


class CrystalInvasionWipeAllTheThingsss(ServerPacket):
    id = MessageID.CrystalInvasionWipeAllTheThingsss

    def read(self, reader):
        pass



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class Ping(SyncPacket):
    id = MessageID.Ping

    def read(self, reader: Reader):
        pass

    def write(self, writer: Writer):
        pass



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class ReleaseItemOwnership(ServerPacket):
    id = MessageID.ReleaseItemOwnership

    def read(self, reader: Reader):
        self.item_index = reader.read_ushort()



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class TamperWithNPC(ServerPacket):
    id = MessageID.TamperWithNPC

    def __init__(
        self,
        npc_id: int = 0,
        set_npc_immunity: bool = False,
        immunity_time: int = 0,
        immunity_player_id: int = 0,
    ):
        self.npc_id = npc_id
        self.set_npc_immunity = set_npc_immunity
        self.immunity_time = immunity_time
        self.immunity_player_id = immunity_player_id

    def read(self, reader: Reader) -> None:
        self.npc_id = reader.read_ushort()
        self.set_npc_immunity = bool(reader.read_byte())
        if self.set_npc_immunity:
            self.immunity_time = reader.read_int()
            self.immunity_player_id = reader.read_short()



from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer


@dataclass
class Rgb:
    r: int = 0
    g: int = 0
    b: int = 0
    a: int = 0

    def __init__(
        self, r: int | None = None, g: int | None = None, b: int | None = None, a: int | None = None
    ) -> None:
        # new Color()
        if r is None and g is None and b is None and a is None:
            self.r = 0
            self.g = 0
            self.b = 0
            self.a = 0
            return

        # new Color(r,g,b)
        if r is not None and g is not None and b is not None and a is None:
            self.r = r
            self.g = g
            self.b = b
            self.a = 255
            return

        # new Color(r,g,b,a)
        if r is not None and g is not None and b is not None and a is not None:
            self.r = r
            self.g = g
            self.b = b
            self.a = a
            return

        raise ValueError("Invalid constructor arguments")

    def get_R(self) -> int:
        return self.r

    def get_G(self) -> int:
        return self.g

    def get_B(self) -> int:
        return self.b

    def get_A(self) -> int:
        return self.a

    @classmethod
    def get_Transparent(cls):
        return cls(0, 0, 0, 0)

    @classmethod
    def read(cls, reader: Reader) -> 'Rgb':
        return cls(
            reader.read_byte(),
            reader.read_byte(),
            reader.read_byte(),
        )

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.r)
        writer.write_byte(self.g)
        writer.write_byte(self.b)

    def __mul__(self, scalar: int | float) -> 'Rgb':
        return Rgb(int(self.r * scalar), int(self.g * scalar), int(self.b * scalar))

    def __str__(self):
        return f"Rgb(r={self.r}, g={self.g}, b={self.b}, a={self.a})"

    @classmethod
    def get_AliceBlue(cls) -> 'Rgb':
        return cls(240, 248, 255)

    @classmethod
    def get_AntiqueWhite(cls) -> 'Rgb':
        return cls(250, 235, 215)

    @classmethod
    def get_Aqua(cls) -> 'Rgb':
        return cls(0, 255, 255)

    @classmethod
    def get_Aquamarine(cls) -> 'Rgb':
        return cls(127, 255, 212)

    @classmethod
    def get_Azure(cls) -> 'Rgb':
        return cls(240, 255, 255)

    @classmethod
    def get_Beige(cls) -> 'Rgb':
        return cls(245, 245, 220)

    @classmethod
    def get_Bisque(cls) -> 'Rgb':
        return cls(255, 228, 196)

    @classmethod
    def get_Black(cls) -> 'Rgb':
        return cls(0, 0, 0)

    @classmethod
    def get_BlanchedAlmond(cls) -> 'Rgb':
        return cls(255, 235, 205)

    @classmethod
    def get_Blue(cls) -> 'Rgb':
        return cls(0, 0, 255)

    @classmethod
    def get_BlueViolet(cls) -> 'Rgb':
        return cls(138, 43, 226)

    @classmethod
    def get_Brown(cls) -> 'Rgb':
        return cls(165, 42, 42)

    @classmethod
    def get_BurlyWood(cls) -> 'Rgb':
        return cls(222, 184, 135)

    @classmethod
    def get_CadetBlue(cls) -> 'Rgb':
        return cls(95, 158, 160)

    @classmethod
    def get_Chartreuse(cls) -> 'Rgb':
        return cls(127, 255, 0)

    @classmethod
    def get_Chocolate(cls) -> 'Rgb':
        return cls(210, 105, 30)

    @classmethod
    def get_Coral(cls) -> 'Rgb':
        return cls(255, 127, 80)

    @classmethod
    def get_CornflowerBlue(cls) -> 'Rgb':
        return cls(100, 149, 237)

    @classmethod
    def get_Cornsilk(cls) -> 'Rgb':
        return cls(255, 248, 220)

    @classmethod
    def get_Crimson(cls) -> 'Rgb':
        return cls(220, 20, 60)

    @classmethod
    def get_Cyan(cls) -> 'Rgb':
        return cls(0, 255, 255)

    @classmethod
    def get_DarkBlue(cls) -> 'Rgb':
        return cls(0, 0, 139)

    @classmethod
    def get_DarkCyan(cls) -> 'Rgb':
        return cls(0, 139, 139)

    @classmethod
    def get_DarkGoldenrod(cls) -> 'Rgb':
        return cls(184, 134, 11)

    @classmethod
    def get_DarkGray(cls) -> 'Rgb':
        return cls(169, 169, 169)

    @classmethod
    def get_DarkGreen(cls) -> 'Rgb':
        return cls(0, 100, 0)

    @classmethod
    def get_DarkKhaki(cls) -> 'Rgb':
        return cls(189, 183, 107)

    @classmethod
    def get_DarkMagenta(cls) -> 'Rgb':
        return cls(139, 0, 139)

    @classmethod
    def get_DarkOliveGreen(cls) -> 'Rgb':
        return cls(85, 107, 47)

    @classmethod
    def get_DarkOrange(cls) -> 'Rgb':
        return cls(255, 140, 0)

    @classmethod
    def get_DarkOrchid(cls) -> 'Rgb':
        return cls(153, 50, 204)

    @classmethod
    def get_DarkRed(cls) -> 'Rgb':
        return cls(139, 0, 0)

    @classmethod
    def get_DarkSalmon(cls) -> 'Rgb':
        return cls(233, 150, 122)

    @classmethod
    def get_DarkSeaGreen(cls) -> 'Rgb':
        return cls(143, 188, 143)

    @classmethod
    def get_DarkSlateBlue(cls) -> 'Rgb':
        return cls(72, 61, 139)

    @classmethod
    def get_DarkSlateGray(cls) -> 'Rgb':
        return cls(47, 79, 79)

    @classmethod
    def get_DarkTurquoise(cls) -> 'Rgb':
        return cls(0, 206, 209)

    @classmethod
    def get_DarkViolet(cls) -> 'Rgb':
        return cls(148, 0, 211)

    @classmethod
    def get_DeepPink(cls) -> 'Rgb':
        return cls(255, 20, 147)

    @classmethod
    def get_DeepSkyBlue(cls) -> 'Rgb':
        return cls(0, 191, 255)

    @classmethod
    def get_DimGray(cls) -> 'Rgb':
        return cls(105, 105, 105)

    @classmethod
    def get_DodgerBlue(cls) -> 'Rgb':
        return cls(30, 144, 255)

    @classmethod
    def get_Firebrick(cls) -> 'Rgb':
        return cls(178, 34, 34)

    @classmethod
    def get_FloralWhite(cls) -> 'Rgb':
        return cls(255, 250, 240)

    @classmethod
    def get_ForestGreen(cls) -> 'Rgb':
        return cls(34, 139, 34)

    @classmethod
    def get_Fuchsia(cls) -> 'Rgb':
        return cls(255, 0, 255)

    @classmethod
    def get_Gainsboro(cls) -> 'Rgb':
        return cls(220, 220, 220)

    @classmethod
    def get_GhostWhite(cls) -> 'Rgb':
        return cls(248, 248, 255)

    @classmethod
    def get_Gold(cls) -> 'Rgb':
        return cls(255, 215, 0)

    @classmethod
    def get_Goldenrod(cls) -> 'Rgb':
        return cls(218, 165, 32)

    @classmethod
    def get_Gray(cls) -> 'Rgb':
        return cls(128, 128, 128)

    @classmethod
    def get_Green(cls) -> 'Rgb':
        return cls(0, 128, 0)

    @classmethod
    def get_GreenYellow(cls) -> 'Rgb':
        return cls(173, 255, 47)

    @classmethod
    def get_Honeydew(cls) -> 'Rgb':
        return cls(240, 255, 240)

    @classmethod
    def get_HotPink(cls) -> 'Rgb':
        return cls(255, 105, 180)

    @classmethod
    def get_IndianRed(cls) -> 'Rgb':
        return cls(205, 92, 92)

    @classmethod
    def get_Indigo(cls) -> 'Rgb':
        return cls(75, 0, 130)

    @classmethod
    def get_Ivory(cls) -> 'Rgb':
        return cls(255, 255, 240)

    @classmethod
    def get_Khaki(cls) -> 'Rgb':
        return cls(240, 230, 140)

    @classmethod
    def get_Lavender(cls) -> 'Rgb':
        return cls(230, 230, 250)

    @classmethod
    def get_LavenderBlush(cls) -> 'Rgb':
        return cls(255, 240, 245)

    @classmethod
    def get_LawnGreen(cls) -> 'Rgb':
        return cls(124, 252, 0)

    @classmethod
    def get_LemonChiffon(cls) -> 'Rgb':
        return cls(255, 250, 205)

    @classmethod
    def get_LightBlue(cls) -> 'Rgb':
        return cls(173, 216, 230)

    @classmethod
    def get_LightCoral(cls) -> 'Rgb':
        return cls(240, 128, 128)

    @classmethod
    def get_LightCyan(cls) -> 'Rgb':
        return cls(224, 255, 255)

    @classmethod
    def get_LightGoldenrodYellow(cls) -> 'Rgb':
        return cls(250, 250, 210)

    @classmethod
    def get_LightGray(cls) -> 'Rgb':
        return cls(211, 211, 211)

    @classmethod
    def get_LightGreen(cls) -> 'Rgb':
        return cls(144, 238, 144)

    @classmethod
    def get_LightPink(cls) -> 'Rgb':
        return cls(255, 182, 193)

    @classmethod
    def get_LightSalmon(cls) -> 'Rgb':
        return cls(255, 160, 122)

    @classmethod
    def get_LightSeaGreen(cls) -> 'Rgb':
        return cls(32, 178, 170)

    @classmethod
    def get_LightSkyBlue(cls) -> 'Rgb':
        return cls(135, 206, 250)

    @classmethod
    def get_LightSlateGray(cls) -> 'Rgb':
        return cls(119, 136, 153)

    @classmethod
    def get_LightSteelBlue(cls) -> 'Rgb':
        return cls(176, 196, 222)

    @classmethod
    def get_LightYellow(cls) -> 'Rgb':
        return cls(255, 255, 224)

    @classmethod
    def get_Lime(cls) -> 'Rgb':
        return cls(0, 255, 0)

    @classmethod
    def get_LimeGreen(cls) -> 'Rgb':
        return cls(50, 205, 50)

    @classmethod
    def get_Linen(cls) -> 'Rgb':
        return cls(250, 240, 230)

    @classmethod
    def get_Magenta(cls) -> 'Rgb':
        return cls(255, 0, 255)

    @classmethod
    def get_Maroon(cls) -> 'Rgb':
        return cls(128, 0, 0)

    @classmethod
    def get_MediumAquamarine(cls) -> 'Rgb':
        return cls(102, 205, 170)

    @classmethod
    def get_MediumBlue(cls) -> 'Rgb':
        return cls(0, 0, 205)

    @classmethod
    def get_MediumOrchid(cls) -> 'Rgb':
        return cls(186, 85, 211)

    @classmethod
    def get_MediumPurple(cls) -> 'Rgb':
        return cls(147, 112, 219)

    @classmethod
    def get_MediumSeaGreen(cls) -> 'Rgb':
        return cls(60, 179, 113)

    @classmethod
    def get_MediumSlateBlue(cls) -> 'Rgb':
        return cls(123, 104, 238)

    @classmethod
    def get_MediumSpringGreen(cls) -> 'Rgb':
        return cls(0, 250, 154)

    @classmethod
    def get_MediumTurquoise(cls) -> 'Rgb':
        return cls(72, 209, 204)

    @classmethod
    def get_MediumVioletRed(cls) -> 'Rgb':
        return cls(199, 21, 133)

    @classmethod
    def get_MidnightBlue(cls) -> 'Rgb':
        return cls(25, 25, 112)

    @classmethod
    def get_MintCream(cls) -> 'Rgb':
        return cls(245, 255, 250)

    @classmethod
    def get_MistyRose(cls) -> 'Rgb':
        return cls(255, 228, 225)

    @classmethod
    def get_Moccasin(cls) -> 'Rgb':
        return cls(255, 228, 181)

    @classmethod
    def get_NavajoWhite(cls) -> 'Rgb':
        return cls(255, 222, 173)

    @classmethod
    def get_Navy(cls) -> 'Rgb':
        return cls(0, 0, 128)

    @classmethod
    def get_OldLace(cls) -> 'Rgb':
        return cls(253, 245, 230)

    @classmethod
    def get_Olive(cls) -> 'Rgb':
        return cls(128, 128, 0)

    @classmethod
    def get_OliveDrab(cls) -> 'Rgb':
        return cls(107, 142, 35)

    @classmethod
    def get_Orange(cls) -> 'Rgb':
        return cls(255, 165, 0)

    @classmethod
    def get_OrangeRed(cls) -> 'Rgb':
        return cls(255, 69, 0)

    @classmethod
    def get_Orchid(cls) -> 'Rgb':
        return cls(218, 112, 214)

    @classmethod
    def get_PaleGoldenrod(cls) -> 'Rgb':
        return cls(238, 232, 170)

    @classmethod
    def get_PaleGreen(cls) -> 'Rgb':
        return cls(152, 251, 152)

    @classmethod
    def get_PaleTurquoise(cls) -> 'Rgb':
        return cls(175, 238, 238)

    @classmethod
    def get_PaleVioletRed(cls) -> 'Rgb':
        return cls(219, 112, 147)

    @classmethod
    def get_PapayaWhip(cls) -> 'Rgb':
        return cls(255, 239, 213)

    @classmethod
    def get_PeachPuff(cls) -> 'Rgb':
        return cls(255, 218, 185)

    @classmethod
    def get_Peru(cls) -> 'Rgb':
        return cls(205, 133, 63)

    @classmethod
    def get_Pink(cls) -> 'Rgb':
        return cls(255, 192, 203)

    @classmethod
    def get_Plum(cls) -> 'Rgb':
        return cls(221, 160, 221)

    @classmethod
    def get_PowderBlue(cls) -> 'Rgb':
        return cls(176, 224, 230)

    @classmethod
    def get_Purple(cls) -> 'Rgb':
        return cls(128, 0, 128)

    @classmethod
    def get_Red(cls) -> 'Rgb':
        return cls(255, 0, 0)

    @classmethod
    def get_RosyBrown(cls) -> 'Rgb':
        return cls(188, 143, 143)

    @classmethod
    def get_RoyalBlue(cls) -> 'Rgb':
        return cls(65, 105, 225)

    @classmethod
    def get_SaddleBrown(cls) -> 'Rgb':
        return cls(139, 69, 19)

    @classmethod
    def get_Salmon(cls) -> 'Rgb':
        return cls(250, 128, 114)

    @classmethod
    def get_SandyBrown(cls) -> 'Rgb':
        return cls(244, 164, 96)

    @classmethod
    def get_SeaGreen(cls) -> 'Rgb':
        return cls(46, 139, 87)

    @classmethod
    def get_SeaShell(cls) -> 'Rgb':
        return cls(255, 245, 238)

    @classmethod
    def get_Sienna(cls) -> 'Rgb':
        return cls(160, 82, 45)

    @classmethod
    def get_Silver(cls) -> 'Rgb':
        return cls(192, 192, 192)

    @classmethod
    def get_SkyBlue(cls) -> 'Rgb':
        return cls(135, 206, 235)

    @classmethod
    def get_SlateBlue(cls) -> 'Rgb':
        return cls(106, 90, 205)

    @classmethod
    def get_SlateGray(cls) -> 'Rgb':
        return cls(112, 128, 144)

    @classmethod
    def get_Snow(cls) -> 'Rgb':
        return cls(255, 250, 250)

    @classmethod
    def get_SpringGreen(cls) -> 'Rgb':
        return cls(0, 255, 127)

    @classmethod
    def get_SteelBlue(cls) -> 'Rgb':
        return cls(70, 130, 180)

    @classmethod
    def get_Tan(cls) -> 'Rgb':
        return cls(210, 180, 140)

    @classmethod
    def get_Teal(cls) -> 'Rgb':
        return cls(0, 128, 128)

    @classmethod
    def get_Thistle(cls) -> 'Rgb':
        return cls(216, 191, 216)

    @classmethod
    def get_Tomato(cls) -> 'Rgb':
        return cls(255, 99, 71)

    @classmethod
    def get_Turquoise(cls) -> 'Rgb':
        return cls(64, 224, 208)

    @classmethod
    def get_Violet(cls) -> 'Rgb':
        return cls(238, 130, 238)

    @classmethod
    def get_Wheat(cls) -> 'Rgb':
        return cls(245, 222, 179)

    @classmethod
    def get_White(cls) -> 'Rgb':
        return cls(255, 255, 255)

    @classmethod
    def get_WhiteSmoke(cls) -> 'Rgb':
        return cls(245, 245, 245)

    @classmethod
    def get_Yellow(cls) -> 'Rgb':
        return cls(255, 255, 0)

    @classmethod
    def get_YellowGreen(cls) -> 'Rgb':
        return cls(154, 205, 50)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class PlaceObject(SyncPacket):
    id = MessageID.PlaceObject

    def __init__(
        self,
        x: int = 0,
        y: int = 0,
        ty: int = 0,
        style: int = 0,
        alternate: int = 0,
        random: int = 0,
        direction: bool = False,
    ):
        self.x = x
        self.y = y
        self.ty = ty
        self.style = style
        self.alternate = alternate
        self.random = random
        self.direction = direction

    def read(self, reader: Reader):
        self.x = reader.read_short()
        self.y = reader.read_short()
        self.ty = reader.read_short()
        self.style = reader.read_short()
        self.alternate = reader.read_byte()
        self.random = reader.read_sbyte()
        self.direction = reader.read_bool()

    def write(self, writer: Writer):
        writer.write_short(self.x)
        writer.write_short(self.y)
        writer.write_short(self.ty)
        writer.write_short(self.style)
        writer.write_byte(self.alternate)
        writer.write_sbyte(self.random)
        writer.write_bool(self.direction)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class QuickStackChests(ClientPacket):
    id = MessageID.QuickStackChests

    def __init__(self, inventory_slot: int = 0):
        self.inventory_slot = inventory_slot

    def read(self, reader: Reader) -> None:
        self.inventory_slot = reader.read_byte()

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.inventory_slot)



from collections.abc import Callable

from terrex.event.context import EventContext
from terrex.event.types import BlockedEvent, ChatEvent, ItemOwnerChangedEvent, LoginEvent

from .base import E, EventFilter, EventTypeFilter


class PlayerFilter(EventFilter[E]):
    _event_type: type[E]

    def __init__(self, event_type: type[E], predicate: Callable[[E, EventContext], bool]):
        self._event_type = event_type
        self.predicate = predicate

    def matches(self, ctx: EventContext) -> E | None:
        event = ctx.event
        if not isinstance(event, self._event_type):
            return None
        if not self.predicate(event, ctx):
            return None
        return event


# Player Fabrics
def OutgoingMessage() -> PlayerFilter[ChatEvent]:
    return PlayerFilter(ChatEvent, lambda e, ctx: e.player_id == ctx.player.id)


def IncomingMessage(player_id: int | None = None) -> PlayerFilter[ChatEvent]:
    return PlayerFilter(
        ChatEvent,
        lambda e, ctx: e.player_id != ctx.player.id
        and (True if player_id is None else e.player_id == player_id),
    )


def ItemOwnedByMe() -> PlayerFilter[ItemOwnerChangedEvent]:
    return PlayerFilter(ItemOwnerChangedEvent, lambda e, ctx: e.player_id == ctx.player.id)


def ItemOwnedByOther(player_id: int | None = None) -> PlayerFilter[ItemOwnerChangedEvent]:
    return PlayerFilter(
        ItemOwnerChangedEvent,
        lambda e, ctx: e.player_id != ctx.player.id
        and (True if player_id is None else e.player_id == player_id),
    )


# Other Player Filters
def LoginPlayer() -> EventTypeFilter[LoginEvent]:
    return EventTypeFilter(LoginEvent)


def BlockPlayer() -> EventTypeFilter[BlockedEvent]:
    return EventTypeFilter(BlockedEvent)



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class SyncRevengeMarker(ServerPacket):
    id = MessageID.SyncRevengeMarker

    def __init__(
        self,
        unique_id: int = 0,
        x: float = 0.0,
        y: float = 0.0,
        npc_id: int = 0,
        npc_hp_percent: float = 0.0,
        npc_type: int = 0,
        npc_ai: int = 0,
        coin_value: int = 0,
        base_value: float = 0.0,
        spawned_from_statue: bool = False,
    ):
        self.unique_id = unique_id
        self.x = x
        self.y = y
        self.npc_id = npc_id
        self.npc_hp_percent = npc_hp_percent
        self.npc_type = npc_type
        self.npc_ai = npc_ai
        self.coin_value = coin_value
        self.base_value = base_value
        self.spawned_from_statue = spawned_from_statue

    def read(self, reader: Reader) -> None:
        self.unique_id = reader.read_int()
        self.x = reader.read_float()
        self.y = reader.read_float()
        self.npc_id = reader.read_int()
        self.npc_hp_percent = reader.read_float()
        self.npc_type = reader.read_int()
        self.npc_ai = reader.read_int()
        self.coin_value = reader.read_int()
        self.base_value = reader.read_float()
        self.spawned_from_statue = reader.read_bool()



from terrex.event.types import ItemOwnerChangedEvent
from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class ItemOwner(SyncPacket):
    id = MessageID.ItemOwner

    def __init__(self, item_id: int = 0, player_id: int = 0):
        self.item_id = item_id
        self.player_id = player_id

    def read(self, reader: Reader):
        self.item_id = reader.read_short()
        self.player_id = reader.read_byte()

    def write(self, writer: Writer):
        writer.write_short(self.item_id)
        writer.write_byte(self.player_id)

    async def handle(self, world, player, evman):
        world.item_owner_index[self.item_id] = self.player_id
        evman.raise_event(ItemOwnerChangedEvent(self, self.item_id, self.player_id))



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import SyncPacket


class SyncProjectile(SyncPacket):
    id = MessageID.SyncProjectile

    def __init__(
        self,
        projectile_id: int = 0,
        pos: Vec2 | None = None,
        vel: Vec2 | None = None,
        owner: int = 0,
        ty: int = 0,
        flags: int = 0,
        ai: list[float] | None = None,
        damage: int | None = None,
        knockback: float | None = None,
        original_damage: int | None = None,
        proj_uuid: int | None = None,
    ):
        self.projectile_id = projectile_id
        self.pos = pos or Vec2(0.0, 0.0)
        self.vel = vel or Vec2(0.0, 0.0)
        self.owner = owner
        self.ty = ty
        self.flags = flags
        self.ai = ai or [0.0] * 2
        self.damage = damage
        self.knockback = knockback
        self.original_damage = original_damage
        self.proj_uuid = proj_uuid

    def read(self, reader: Reader):
        self.projectile_id = reader.read_short()
        self.pos = Vec2.read(reader)
        self.vel = Vec2.read(reader)
        self.owner = reader.read_byte()
        self.ty = reader.read_short()
        self.flags = reader.read_byte()
        ai_flags = [0x01, 0x02]
        for i in range(2):
            if self.flags & ai_flags[i]:
                self.ai[i] = reader.read_float()
        if self.flags & 0x10:  # HAS_DAMAGE
            self.damage = reader.read_short()
        if self.flags & 0x20:  # HAS_KNOCKBACK
            self.knockback = reader.read_float()
        if self.flags & 0x40:  # HAS_ORIG_DAMAGE
            self.original_damage = reader.read_short()
        if self.flags & 0x80:  # HAS_UUID
            self.proj_uuid = reader.read_short()

    def write(self, writer: Writer):
        writer.write_short(self.projectile_id)
        self.pos.write(writer)
        self.vel.write(writer)
        writer.write_byte(self.owner)
        writer.write_short(self.ty)
        writer.write_byte(self.flags)
        ai_flags = [0x01, 0x02]
        for i in range(2):
            if self.flags & ai_flags[i]:
                writer.write_float(self.ai[i])
        if self.damage is not None:
            writer.write_short(self.damage)
        if self.knockback is not None:
            writer.write_float(self.knockback)
        if self.original_damage is not None:
            writer.write_short(self.original_damage)
        if self.proj_uuid is not None:
            writer.write_short(self.proj_uuid)



"""
Tile entity module for Terrex. Contains classes for Terraria tile entities.
"""

from .tile_entity import (
    CritterAnchor,
    DeadCellsDisplayJar,
    DisplayDoll,
    FoodPlatter,
    HatRack,
    ItemFrame,
    KiteAnchor,
    LeashedEntityAnchorWithItem,
    LogicSensor,
    TeleportationPylon,
    TileEntity,
    TileEntityBase,
    TrainingDummy,
    WeaponRack,
    read_tile_entity,
)

__all__ = [
    "TileEntityBase",
    "TrainingDummy",
    "ItemFrame",
    "LogicSensor",
    "DisplayDoll",
    "WeaponRack",
    "HatRack",
    "FoodPlatter",
    "TeleportationPylon",
    "DeadCellsDisplayJar",
    "LeashedEntityAnchorWithItem",
    "KiteAnchor",
    "CritterAnchor",
    "TileEntity",
    "read_tile_entity",
]



from enum import IntEnum

from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class DoorAction(IntEnum):
    OPEN_DOOR = 0
    CLOSE_DOOR = 1
    OPEN_TRAPDOOR = 2
    CLOSE_TRAPDOOR = 3
    OPEN_TALL_GATE = 4
    CLOSE_TALL_GATE = 5

    @classmethod
    def read(cls, reader: Reader) -> 'DoorAction':
        return cls(reader.read_byte())

    def write(self, writer: Writer) -> None:
        writer.write_byte(self.value)


class ToggleDoorState(SyncPacket):
    id = MessageID.ToggleDoorState

    def __init__(
        self,
        action: DoorAction = DoorAction.OPEN_DOOR,
        tile_x: int = 0,
        tile_y: int = 0,
        direction: int = 0,
    ):
        self.action = action
        self.tile_x = tile_x
        self.tile_y = tile_y
        self.direction = direction

    def read(self, reader: Reader):
        self.action = DoorAction.read(reader)
        self.tile_x = reader.read_short()
        self.tile_y = reader.read_short()
        self.direction = reader.read_sbyte()

    def write(self, writer: Writer):
        self.action.write(writer)
        writer.write_short(self.tile_x)
        writer.write_short(self.tile_y)
        writer.write_sbyte(self.direction)



from .any_var import AnyVar
from .bool_var import BoolVar
from .const_var import ConstVar
from .float_var import FloatVar
from .int_var import IntVar
from .str_var import StrVar

__all__ = [
    "AnyVar",
    "BoolVar",
    "ConstVar",
    "FloatVar",
    "IntVar",
    "StrVar",
]



"""
Ported from Terraria source: Terraria.WorldBuilding.Shapes.Slime

Slime-like blob shape: rounded top, tapered bottom.
"""

from math import sqrt
from typing import cast

import numpy as np

from .base import GenAction, GenActionBulk, GenShape, Point


class Slime(GenShape):
    """Slime blob shape with scalable X/Y dimensions."""

    def __init__(
        self, radius: int, x_scale: float = 1.0, y_scale: float = 1.0, quit_on_fail: bool = False
    ):
        super().__init__(quit_on_fail)
        self._radius = radius
        self._xScale = x_scale
        self._yScale = y_scale

    def perform(self, origin: Point, action: GenAction) -> bool:
        num = float(self._radius)
        num1 = (self._radius + 1) ** 2
        use_bulk = isinstance(action, GenActionBulk) and not self._quitOnFail

        xs_all = []
        ys_all = []

        # Upper part (full ellipse)
        for i in range(origin.y - int(num * self._yScale), origin.y + 1):
            y = (i - origin.y) / self._yScale
            width = int(min(self._radius * self._xScale, self._xScale * sqrt(num1 - y * y)))
            if use_bulk:
                xs_all.append(np.arange(origin.x - width, origin.x + width + 1))
                ys_all.append(np.full(2 * width + 1, i))
            else:
                for j in range(origin.x - width, origin.x + width + 1):
                    if not self.unit_apply(action, origin, j, i) and self._quitOnFail:
                        return False

        # Lower part (tapered, half height)
        for k in range(origin.y + 1, origin.y + int(num * self._yScale * 0.5)):
            y1 = (k - origin.y) * (2.0 / self._yScale)
            width = int(
                min(self._radius * self._xScale, self._xScale * sqrt(max(0, num1 - y1 * y1)))
            )
            if use_bulk:
                xs_all.append(np.arange(origin.x - width, origin.x + width + 1))
                ys_all.append(np.full(2 * width + 1, k))
            else:
                for a in range(origin.x - width, origin.x + width + 1):
                    if not self.unit_apply(action, origin, a, k) and self._quitOnFail:
                        return False

        if use_bulk and xs_all:
            xs = np.concatenate(xs_all)
            ys = np.concatenate(ys_all)
            return cast(bool, cast(GenActionBulk, action).apply_bulk(xs, ys))

        return True



from terrex.id import MessageID
from terrex.localization.network_text import NetworkText
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.rgb import Rgb
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import SyncPacket


class CombatTextString(SyncPacket):
    id = MessageID.CombatTextString

    def __init__(
        self,
        pos: Vec2 | None = None,
        color: Rgb | None = None,
        combat_text: NetworkText | None = None,
    ):
        self.pos = pos or Vec2(0.0, 0.0)
        self.color = color or Rgb(0, 0, 0)
        self.combat_text = combat_text or NetworkText()

    def read(self, reader: Reader):
        self.pos = Vec2.read(reader)
        self.color = Rgb.read(reader)
        self.combat_text = NetworkText.read(reader)

    def write(self, writer: Writer):
        self.pos.write(writer)
        self.color.write(writer)
        self.combat_text.write(writer)



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class SetTime(ServerPacket):
    id = MessageID.SetTime

    def __init__(
        self, day_time: bool = False, time: int = 0, sun_mod_y: int = 0, moon_mod_y: int = 0
    ):
        self.day_time = day_time
        self.time = time
        self.sun_mod_y = sun_mod_y
        self.moon_mod_y = moon_mod_y

    def read(self, reader: Reader):
        self.day_time = reader.read_bool()
        self.time = reader.read_int()
        self.sun_mod_y = reader.read_short()
        self.moon_mod_y = reader.read_short()



"""
GlowMaskID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class GlowMaskID(IntEnum):
    FakeUser = -2
    NoneValue = -1
    UFOMinion = 0
    VortexAxe = 1
    VortexChainsawProjectile = 2
    VortexDrillProjectile = 3
    VortexHammer = 4
    VortexPickaxe = 5
    NebulaAxe = 6
    NebulaChainsawProjectile = 7
    NebulaDrillProjectile = 8
    NebulaHammer = 9
    NebulaPickaxe = 10
    MartianOfficer = 11
    NebulaArmorArm = 12
    VortexArmorBody = 13
    NebulaArmorBody = 14
    VortexArmorHead = 15
    NebulaArmorHead = 16
    NebulaArmorLegs = 17
    VortexArmorBodyFemale = 18
    NebulaArmorBodyFemale = 19
    VortexChainsawItem = 20
    VortexDrillItem = 21
    NebulaChainsawItem = 22
    NebulaDrillItem = 23
    BrainScrambler = 24
    Electrified = 25
    VortexItemHead = 26
    VortexItemBody = 27
    NebulaItemHead = 28
    NebulaItemBody = 29
    NebulaItemLegs = 30
    MartianEngineer = 31
    MartianTurret = 32
    MartianDrone = 33
    GigaZapper = 34
    LaserMachinegun = 35
    ElectrosphereLauncher = 36
    ElectrosphereMissile = 37
    Xenopopper = 38
    LaserDrill = 39
    LaserDrillTip = 40
    MartianUniformArmorHead = 41
    MartianUniformArmorBody = 42
    MartianUniformArmorBodyFemale = 43
    MartianUniformArmorArms = 44
    MartianUnifromTorso = 45
    MartianUniformHelmet = 46
    LaserMachinegunItem = 47
    MartianSaucer = 48
    MartianSaucerCore = 49
    MartianSaucerCannon = 50
    MartianAstroClock = 51
    MartianBathtub = 52
    MartianBed = 53
    MartianHoverChair = 54
    MartianChandelier = 55
    MartianChest = 56
    MartianDoorClosed = 57
    MartianDoorOpen = 58
    MartianDresser = 59
    MartianHolobookcase = 60
    MartianHoverCandle = 61
    MartianLamppost = 62
    MartianLantern = 63
    MartianPiano = 64
    MartianPlatform = 65
    MartianSofa = 66
    MartianTable = 67
    MartianTableLamp = 68
    MartianWorkBench = 69
    MartianAstroClockItem = 70
    MartianBathtubItem = 71
    MartianBedItem = 72
    MartianHoverChairItem = 73
    MartianChandelierItem = 74
    MartianChestItem = 75
    MartianDoorItem = 76
    MartianDresserItem = 77
    MartianHolobookcaseItem = 78
    MartianHoverCandleItem = 79
    MartianLamppostItem = 80
    MartianLanternItem = 81
    MartianPianoItem = 82
    MartianPlatformItem = 83
    MartianSofaItem = 84
    MartianTableItem = 85
    MartianTableLampItem = 86
    MartianWorkBenchItem = 87
    MartianSink = 88
    MartianSinkItem = 89
    BGMartianUFOSmall = 90
    BGMartianUFOBig = 91
    MothronWings = 92
    MartianConduitPlatingItem = 93
    MartianConduitPlatingTile = 94
    MartianConduitWallItem = 95
    HiTekSunglasses = 96
    HiTekSunglassesItem = 97
    MartianHairDye = 98
    MartianArmorDye = 99
    MartianProbe = 100
    MartianProbeDiode = 101
    ChargedBlasterCannon = 102
    ChlorophyteDye = 103
    PixieDye = 104
    WispDye = 105
    InfernalWispDye = 106
    UnicornWispDye = 107
    CultistTabletFront = 108
    CultistTabletBack = 109
    ShadowflameApparation = 110
    MeteorBrick = 111
    MeteorPlatform = 112
    MeteorBathtub = 113
    MeteorBed = 114
    MeteorBookcase = 115
    MeteorChair = 116
    MeteorChest = 117
    MeteorClock = 118
    MeteorDoorOpen = 119
    MeteorDresser = 120
    MeteorPiano = 121
    MeteorSink = 122
    MeteorSofa = 123
    MeteorTable = 124
    MeteorWorkbench = 125
    LavaMoss = 126
    LongLavaMoss = 127
    DeadlySphere = 128
    LavaLamp = 129
    LavaLampTile = 130
    EnchantedNightcrawler = 131
    LunarTowerStardust = 132
    StardustWormBody = 133
    StardustWormTail = 134
    StardustWormHead = 135
    StardustSoldier = 136
    StardustSpiderSmall = 137
    StardustSpiderBig = 138
    StardustJellyfishBig = 139
    StardustJellyfishSmall = 140
    StardustCellBig = 141
    StardustCellSmall = 142
    LunarTowerNebula = 143
    NebulaSoldier = 144
    NebulaBeast = 145
    NebulaHeadcrab = 146
    NebulaBrain = 147
    NebulaEye = 148
    LunarTowerVortex = 149
    VortexRifleman = 150
    VortexSoldier = 151
    VortexHornet = 152
    VortexHornetQueen = 153
    SolarSolenian = 154
    SolarDrakoMire = 155
    SolarDrakoMireRider = 156
    SolarCrawltipedeHead = 157
    SolarCrawltipedeBody = 158
    SolarCrawltipedeTail = 159
    SolarSroller = 160
    SolarCorite = 161
    LunarTowerSolar = 162
    SolarSpearman = 163
    MartianWalker = 164
    MartianWalkerCannon = 165
    Truffle = 166
    Cyborg = 167
    TruffleSpore = 168
    DesertGhoulCorruption = 169
    DesertGhoulCrimson = 170
    DesertGhoulHallow = 171
    DesertDjinn = 172
    PortalGunProj = 173
    StardustAxeItem = 174
    StardustChainsawItem = 175
    StardustDrillItem = 176
    StardustHammerItem = 177
    StardustPickaxeItem = 178
    StardustChainsawProjectile = 179
    StardustDrillProjectile = 180
    WingsVortex = 181
    WingsNebula = 182
    WingsStardust = 183
    ArmorStardustHead = 184
    ArmorStardustBodyMale = 185
    ArmorStardustBodyFemale = 186
    ArmorStardustLegs = 187
    ArmorStardustArm = 188
    StardustCellMinion = 189
    StardustCellMinionShot = 190
    VortexBeaterItem = 191
    VortexBeaterProj = 192
    VortexBeaterRocket = 193
    NebulaArcanumItem = 194
    NebulaArcanumItemHold = 195
    LunarHamaxeVortex = 196
    LunarHamaxeNebula = 197
    LunarHamaxeStardust = 198
    VortexArmorLegs = 199
    Phantasm = 200
    LunarMonolith = 201
    LunarCraftingStation = 202
    LunarHookSolar = 203
    LunarHookVortex = 204
    LunarHookNebula = 205
    LunarHookStardust = 206
    NebulaBlaze = 207
    JimsHead = 208
    LokisHelm = 209
    LokisBody = 210
    LokisArm = 211
    LokisLegs = 212
    LokisWings = 213
    WireBulb = 214
    CannonTile = 215
    SandElemental = 216
    ForbiddenSign = 217
    SpiritFlameItem = 218
    BlackBolt = 219
    BlackBoltEye = 220
    DD2FlameBurstTowerT1 = 221
    DD2FlameBurstTowerT2 = 222
    DD2FlameBurstTowerT3 = 223
    DD2WitherBeast = 224
    DD2DarkMage = 225
    DD2Betsy = 226
    DD2SquireBetsySword = 227
    DD2LightningAuraT1 = 228
    DD2LightningAuraT2 = 229
    DD2LightningAuraT3 = 230
    MonkStaffT2 = 231
    MonkStaffT3 = 232
    MonkStaffT3Item = 233
    BetsyBow = 234
    DD2ExplosiveTrapT1 = 235
    DD2ExplosiveTrapT2 = 236
    DD2ExplosiveTrapT3 = 237
    BetsysWrath = 238
    DD2CrystalOrb = 239
    ApprenticeDarkArmFlame = 240
    ApprenticeDarkHeadFlame = 241
    ValhallaKnightHead = 242
    DefendersForge = 243
    DefendersForgeItem = 244
    ArkhalisHat = 245
    ArkhalisShirt_Male = 246
    ArkhalisShirt_Female = 247
    ArkhalisShirt_Arm = 248
    ArkhalisPants_Male = 249
    ArkhalisPants_Female = 250
    ArkhalisWings = 251
    BoulderStaffOfEarth = 252
    Everscream = 253
    Santank = 254
    HeadlessHorseman = 255
    UltraBrightHelmet = 256
    JackOLantern = 257
    LavaMossBrick = 258
    KryptonMoss = 259
    KryptonMossBrick = 260
    XenonMoss = 261
    XenonMossBrick = 262
    ArgonMoss = 263
    ArgonMossBrick = 264
    VoidMonolith = 265
    ScarabBomb = 266
    PlasmaLamp = 267
    PinkFairyJar = 268
    GreenFairyJar = 269
    BlueFairyJar = 270
    FirstFractal = 271
    GroxTheGreatWings = 272
    GroxTheGreatHelm = 273
    GroxTheGreatGreaves = 274
    DestroyerPet = 275
    SkeletronPrimePet = 276
    GolemPet = 277
    PumpkingPet = 278
    EverscreamPet = 279
    FairyQueenPet = 280
    MartianPet = 281
    MoonLordPet = 282
    SpookyWoodMount = 283
    WallOfFleshGoatMount = 284
    LavaSharkMount = 285
    GemSquirrel = 286
    GemBunny = 287
    HellButterfly = 288
    HellButterflyJar = 289
    Lavafly = 290
    LavaflyinaBottle = 291
    MagmaSnail = 292
    MagmaSnailCage = 293
    BrazierSuspended = 294
    VolcanoSmall = 295
    VolcanoLarge = 296
    PottedPlants2 = 297
    PottedLavaPlants = 298
    PottedLavaPlantTendrils = 299
    TruffleWormCage = 300
    RoninHat = 301
    TimelessTravelerHood = 302
    TimelessTravelerBottom = 303
    FloretProtectorHelmet = 304
    CapricornMask = 305
    CapricornLegs = 306
    CapricornTail = 307
    FloretProtectorFrontTube = 308
    TVHeadMask = 309
    Magiluminescence = 310
    VioletMoss = 311
    VioletMossBrick = 312
    RainbowMoss = 313
    RainbowMossBrick = 314
    TreeAsh = 315
    TreeAshTop = 316
    TreeAshBranches = 317
    FishingBobberGlowingStar = 318
    FishingBobberGlowingLava = 319
    FishingBobberGlowingKrypton = 320
    FishingBobberGlowingXenon = 321
    FishingBobberGlowingArgon = 322
    FishingBobberGlowingViolet = 323
    FishingBobberGlowingRainbow = 324
    Sundial = 325
    AshGrass = 326
    AshVines = 327
    LightsBane = 328
    GlowTulip = 329
    EchoMonolith = 330
    ShimmerCloakFront = 331
    ShimmerCloakBack = 332
    ShimmerMonolith = 333
    ShimmerMonolithOrb = 334
    Moondial = 335
    LavaMossBlock = 336
    ArgonMossBlock = 337
    KryptonMossBlock = 338
    XenonMossBlock = 339
    VioletMossBlock = 340
    RainbowMossBlock = 341
    LavaMossBlockWall = 342
    ArgonMossBlockWall = 343
    KryptonMossBlockWall = 344
    XenonMossBlockWall = 345
    VioletMossBlockWall = 346
    RainbowMossBlockWall = 347
    ShimmerBlock = 348
    ShimmerBrick = 349
    CyborgShimmered = 350
    CyborgShimmeredParty = 351
    TruffleShimmered = 352
    DeadCellsPotionStation = 353
    DeadCellsMushroomBoi = 354
    DeadCellsFlint = 355
    DeadCellsBarrel = 356
    DeadCellsBeheadedHead = 357
    EelWhip = 358
    Shimmerfall = 359
    FireImp = 360
    ShimmerfallWall = 361
    LavaCloud = 362
    ChickenBonesRobeLeg = 363
    ChickenBonesRobeTorso = 364
    ChickenBonesHead = 365
    ChickenBonesWings = 366
    UpgradedMiningHead = 367
    CRTMonolith = 368
    RetroMonolith = 369
    BatMountBody = 370
    NoirMonolith = 371
    RainbowPhaseblade = 372
    RainbowPhasesaber = 373
    TrueCopperShortsword = 374
    KazzymodusHood = 375
    KazzymodusChestpiece = 376
    PalworldDigtoise = 377
    LunasHead = 378




"""
ItemHoldStyleID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class ItemHoldStyleID(IntEnum):
    NoneValue = 0
    HoldFront = 1
    HoldUp = 2
    HoldHeavy = 3
    HoldGolfClub = 4
    HoldGuitar = 5
    HoldLamp = 6
    HoldRadio = 7
    HoldOrb = 8




from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer

from .base import ServerPacket


class NPCBuffs(ServerPacket):
    id = MessageID.NPCBuffs
    MAX_BUFF = 20  # NPC.maxBuffs

    def __init__(self) -> None:
        self.npc_id: int = 0
        self.buffs: list[tuple[int, int]] = []  # (buff_type, buff_time)

    def read(self, reader: Reader) -> None:
        self.npc_id = reader.read_short()
        self.buffs = []
        while len(self.buffs) < self.MAX_BUFF:
            buff_type = reader.read_ushort()
            if buff_type == 0:  # packet every end on 0 ushort
                break  # packet ended
            time = reader.read_ushort()
            self.buffs.append((buff_type, time))

    # not used by client
    def write(self, writer: Writer) -> None:
        writer.write_short(self.npc_id)
        for buff_type, time in self.buffs:
            writer.write_ushort(buff_type)
            writer.write_ushort(time)
        writer.write_ushort(0)



from enum import IntEnum

from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class UnlockType(IntEnum):
    CHEST = 1
    DOOR = 2


class LockAndUnlock(SyncPacket):
    id = MessageID.LockAndUnlock

    def __init__(self, ty: UnlockType = UnlockType.CHEST, x: int = 0, y: int = 0):
        self.ty = ty
        self.x = x
        self.y = y

    def read(self, reader: Reader):
        self.ty = UnlockType(reader.read_byte())
        self.x = reader.read_short()
        self.y = reader.read_short()

    def write(self, writer: Writer):
        writer.write_byte(self.ty)
        writer.write_short(self.x)
        writer.write_short(self.y)



from terrex.id import MessageID
from terrex.net.bits_byte import BitsByte
from terrex.net.enum.teleport_pylon_type import TeleportPylonType
from terrex.net.enum.teleport_type import TeleportType
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import SyncPacket


class TeleportEntityFlags:
    def __init__(
        self,
        server_synced: bool = False,
        player_teleport: bool = False,
        spawn_failed: bool = False,
        has_type_of_pylon: bool = False,
    ):
        self.flags: BitsByte = BitsByte()
        self.server_synced = server_synced
        self.player_teleport = player_teleport
        self.spawn_failed = spawn_failed
        self.has_type_of_pylon = has_type_of_pylon

    @classmethod
    def create(cls, flags: int) -> 'TeleportEntityFlags':
        tp_entity = cls()
        tp_entity.flags = BitsByte(flags)
        return cls()

    @property
    def need_sync(self) -> bool:
        return not (self.server_synced or self.player_teleport)

    @property
    def server_synced(self) -> bool:
        return self.flags[0]

    @server_synced.setter
    def server_synced(self, val: bool):
        self.flags[0] = val

    @property
    def player_teleport(self) -> bool:
        return self.flags[1]

    @player_teleport.setter
    def player_teleport(self, val: bool):
        self.flags[1] = val

    @property
    def spawn_failed(self) -> bool:
        return self.flags[2]

    @spawn_failed.setter
    def spawn_failed(self, val: bool):
        self.flags[2] = val

    @property
    def has_type_of_pylon(self) -> bool:
        return self.flags[3]

    @has_type_of_pylon.setter
    def has_type_of_pylon(self, val: bool):
        self.flags[3] = val

    def __int__(self) -> int:
        return int(self.flags)

    def __repr__(self):
        return (
            f"TeleportFlags(server_synced={self.server_synced}, "
            f"player_teleport={self.player_teleport}, "
            f"spawn_failed={self.spawn_failed}, "
            f"has_type_of_pylon={self.has_type_of_pylon}, "
            f"need_sync={self.need_sync}, "
            f"value={int(self.flags):08b})"
        )


class TeleportEntity(SyncPacket):
    id = MessageID.TeleportEntity

    def __init__(
        self,
        server_synced: bool = False,
        player_teleport: bool = False,
        player_id: int = 0,
        position: Vec2 | None = None,
        type: TeleportType = TeleportType.TeleporterTile,
        pylon_type: TeleportPylonType = TeleportPylonType.SurfacePurity,
    ) -> None:
        self.flags: TeleportEntityFlags = TeleportEntityFlags(
            server_synced,
            player_teleport,
            has_type_of_pylon=pylon_type != TeleportPylonType.SurfacePurity,
        )
        self.player_id: int = player_id
        self.position: Vec2 = position or Vec2(0, 0)
        self.teleport_type: int = type
        self.pylon_type: int = pylon_type

    def read(self, reader: Reader) -> None:
        self.flags = TeleportEntityFlags.create(reader.read_byte())
        self.player_id = reader.read_short()
        self.position.x = reader.read_float()
        self.position.y = reader.read_float()
        self.teleport_type = TeleportType(reader.read_byte())
        if self.flags.has_type_of_pylon:
            self.pylon_type = TeleportPylonType(reader.read_int())

    def write(self, writer: Writer) -> None:
        writer.write_byte(int(self.flags))
        writer.write_short(self.player_id)
        writer.write_float(self.position.x)
        writer.write_float(self.position.y)
        writer.write_byte(self.teleport_type)
        if self.flags.has_type_of_pylon:
            writer.write_int(self.pylon_type)



from .creative_power import CreativePower, creative_power_registry
from .difficulty_slider_power import DifficultySliderPower
from .far_placement_range_power import FarPlacementRangePower
from .freeze_rain_power import FreezeRainPower
from .freeze_time_power import FreezeTimePower
from .freeze_wind_direction_and_strength_power import (
    FreezeWindDirectionAndStrengthPower,
)
from .godmode_power import GodmodePower
from .modify_rain_power import ModifyRainPower
from .modify_time_rate_power import ModifyTimeRatePower
from .modify_wind_direction_and_strength_power import (
    ModifyWindDirectionAndStrengthPower,
)
from .spawn_rate_slider_per_player_power import SpawnRateSliderPerPlayerPower
from .start_day_immediately_power import StartDayImmediatelyPower
from .start_midnight_immediately_power import StartMidnightImmediatelyPower
from .start_night_immediately_power import StartNightImmediatelyPower
from .start_noon_immediately_power import StartNoonImmediatelyPower
from .stop_biome_spread_power import StopBiomeSpreadPower

__all__ = [
    "CreativePower",
    "creative_power_registry",
    "DifficultySliderPower",
    "FarPlacementRangePower",
    "FreezeRainPower",
    "FreezeTimePower",
    "FreezeWindDirectionAndStrengthPower",
    "GodmodePower",
    "ModifyRainPower",
    "ModifyTimeRatePower",
    "ModifyWindDirectionAndStrengthPower",
    "SpawnRateSliderPerPlayerPower",
    "StartDayImmediatelyPower",
    "StartMidnightImmediatelyPower",
    "StartNightImmediatelyPower",
    "StartNoonImmediatelyPower",
    "StopBiomeSpreadPower",
]



from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2

from .net_module import NetSyncModule


@dataclass()
class NetPingModule(NetSyncModule):
    id: int = 2
    pos: Vec2 | None = None

    @classmethod
    def create(cls, pos: Vec2) -> "NetPingModule":
        obj = cls()
        obj.pos = pos
        return obj

    def read(self, reader: Reader) -> None:
        self.pos = Vec2.read(reader)

    def write(self, writer: Writer) -> None:
        if self.pos is None:
            raise ValueError("pos must be Vec2 and not None")
        self.pos.write(writer)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class PlayerStealth(SyncPacket):
    id = MessageID.PlayerStealth

    def __init__(self, player: int = 0, stealth: float = 0.0):
        self.player = player
        self.stealth = stealth

    def read(self, reader: Reader):
        self.player = reader.read_byte()
        self.stealth = reader.read_float()

    def write(self, writer: Writer):
        writer.write_byte(self.player)
        writer.write_float(self.stealth)



from re import Match

from terrex.net.enum.chat_command import ChatCommand


class BaseEvent:
    def __init__(self, packet):
        from terrex.packet import Packet

        if not isinstance(packet, Packet):
            raise TypeError("packet must be a Packet instance")

        self.packet = packet


class RegexEvent(BaseEvent):
    pattern_match: Match[str] | None = None


class LoginEvent(BaseEvent):
    def __init__(self, packet):
        super().__init__(packet)


class PlayerIDEvent(BaseEvent):
    def __init__(self, packet, player_id: int):
        super().__init__(packet)
        self.player_id: int = player_id


class BlockedEvent(BaseEvent):
    def __init__(self, packet, reason: str):
        super().__init__(packet)
        self.reason: str = reason


class InitializedEvent(BaseEvent):
    def __init__(self, packet):
        super().__init__(packet)


class ChatEvent(RegexEvent):
    def __init__(
        self, packet, player_id: int, text: str, chat_command_id: ChatCommand | None = None
    ):
        super().__init__(packet)
        self.player_id: int = player_id
        self.text: str = text
        self.chat_command_id: ChatCommand | None = chat_command_id


class WorldSectionUpdateEvent(BaseEvent):
    def __init__(self, packet, section):
        super().__init__(packet)

        from terrex.net.structure.world_section import WorldSection

        if not isinstance(section, WorldSection):
            return

        self.section = section


class ItemOwnerChangedEvent(BaseEvent):
    def __init__(self, packet, item_id: int, player_id: int):
        super().__init__(packet)
        self.item_id: int = item_id
        self.player_id: int = player_id


class ItemEvent(BaseEvent):
    def __init__(self, packet, item):
        super().__init__(packet)

        from terrex.item.item import Item

        if not isinstance(item, Item):
            raise TypeError("item must be a Item instance")

        self.item: Item = item


class ItemDroppedEvent(ItemEvent):
    pass


class ItemDropUpdateEvent(ItemEvent):
    pass



from dataclasses import dataclass

from terrex.net.creative_power.creative_power import CreativePower
from terrex.net.streamer import Reader, Writer


@dataclass()
class FreezeTimePower(CreativePower):
    id: int = 0
    enabled: bool = False

    @classmethod
    def create(cls, enabled: bool = False) -> "FreezeTimePower":
        obj = cls()
        obj.enabled = enabled
        return obj

    def read(self, reader: Reader) -> None:
        self.enabled = reader.read_bool()

    def write(self, writer: Writer) -> None:
        writer.write_bool(self.enabled)



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class WorldData(ServerPacket):
    id = MessageID.WorldData

    def __init__(
        self,
        time: int = 0,
        day_info: int = 0,
        moon_phase: int = 0,
        max_tiles_x: int = 0,
        max_tiles_y: int = 0,
        spawn_x: int = 0,
        spawn_y: int = 0,
        world_surface: int = 0,
        rock_layer: int = 0,
        world_id: int = 0,
        world_name: str = "",
        game_mode: int = 0,
        world_unique_id: list[int] | None = None,
        world_generator_version: list[int] | None = None,
        moon_type: int = 0,
        tree_background: int = 0,
        corruption_background: int = 0,
        jungle_background: int = 0,
        snow_background: int = 0,
        hallow_background: int = 0,
        crimson_background: int = 0,
        desert_background: int = 0,
        ocean_background: int = 0,
        unknown_background: list[int] | None = None,
        ice_back_style: int = 0,
        jungle_back_style: int = 0,
        hell_back_style: int = 0,
        wind_speed_set: float = 0.0,
        cloud_number: int = 0,
        trees: list[int] | None = None,
        tree_styles: list[int] | None = None,
        cave_backs: list[int] | None = None,
        cave_back_styles: list[int] | None = None,
        forest_tree_top_styles: list[int] | None = None,
        corruption_tree_top_style: int = 0,
        jungle_tree_top_style: int = 0,
        snow_tree_top_style: int = 0,
        hallow_tree_top_style: int = 0,
        crimson_tree_top_style: int = 0,
        desert_tree_top_style: int = 0,
        ocean_tree_top_style: int = 0,
        glowing_mushroom_tree_top_style: int = 0,
        underworld_tree_top_style: int = 0,
        rain: float = 0.0,
        event_info: int = 0,
        ore_tiers_tiles: list[int] | None = None,
        invasion_type: int = 0,
        lobby_id: int = 0,
        sandstorm_severity: float = 0.0,
    ):
        self.time = time
        self.day_info = day_info
        self.moon_phase = moon_phase
        self.max_tiles_x = max_tiles_x
        self.max_tiles_y = max_tiles_y
        self.spawn_x = spawn_x
        self.spawn_y = spawn_y
        self.world_surface = world_surface
        self.rock_layer = rock_layer
        self.world_id = world_id
        self.world_name = world_name
        self.game_mode = game_mode
        self.world_unique_id = world_unique_id or [0] * 16
        self.world_generator_version = world_generator_version or [0, 0]
        self.moon_type = moon_type
        self.tree_background = tree_background
        self.corruption_background = corruption_background
        self.jungle_background = jungle_background
        self.snow_background = snow_background
        self.hallow_background = hallow_background
        self.crimson_background = crimson_background
        self.desert_background = desert_background
        self.ocean_background = ocean_background
        self.unknown_background = unknown_background or [0] * 5
        self.ice_back_style = ice_back_style
        self.jungle_back_style = jungle_back_style
        self.hell_back_style = hell_back_style
        self.wind_speed_set = wind_speed_set
        self.cloud_number = cloud_number
        self.trees = trees or [0, 0, 0]
        self.tree_styles = tree_styles or [0, 0, 0, 0]
        self.cave_backs = cave_backs or [0, 0, 0]
        self.cave_back_styles = cave_back_styles or [0, 0, 0, 0]
        self.forest_tree_top_styles = forest_tree_top_styles or [0, 0, 0, 0]
        self.corruption_tree_top_style = corruption_tree_top_style
        self.jungle_tree_top_style = jungle_tree_top_style
        self.snow_tree_top_style = snow_tree_top_style
        self.hallow_tree_top_style = hallow_tree_top_style
        self.crimson_tree_top_style = crimson_tree_top_style
        self.desert_tree_top_style = desert_tree_top_style
        self.ocean_tree_top_style = ocean_tree_top_style
        self.glowing_mushroom_tree_top_style = glowing_mushroom_tree_top_style
        self.underworld_tree_top_style = underworld_tree_top_style
        self.rain = rain
        self.event_info = event_info
        self.ore_tiers_tiles = ore_tiers_tiles or [0] * 7
        self.invasion_type = invasion_type
        self.lobby_id = lobby_id
        self.sandstorm_severity = sandstorm_severity

    def read(self, reader: Reader):
        self.time = reader.read_int()
        self.day_info = reader.read_byte()
        self.moon_phase = reader.read_byte()
        self.max_tiles_x = reader.read_short()
        self.max_tiles_y = reader.read_short()
        self.spawn_x = reader.read_short()
        self.spawn_y = reader.read_short()
        self.world_surface = reader.read_short()
        self.rock_layer = reader.read_short()
        self.world_id = reader.read_int()
        self.world_name = reader.read_dotnet_string()
        self.game_mode = reader.read_byte()
        self.world_unique_id = [reader.read_byte() for _ in range(16)]
        self.world_generator_version = [reader.read_int(), reader.read_int()]
        self.moon_type = reader.read_byte()
        self.tree_background = reader.read_byte()
        self.corruption_background = reader.read_byte()
        self.jungle_background = reader.read_byte()
        self.snow_background = reader.read_byte()
        self.hallow_background = reader.read_byte()
        self.crimson_background = reader.read_byte()
        self.desert_background = reader.read_byte()
        self.ocean_background = reader.read_byte()
        self.unknown_background = [reader.read_byte() for _ in range(5)]
        self.ice_back_style = reader.read_byte()
        self.jungle_back_style = reader.read_byte()
        self.hell_back_style = reader.read_byte()
        self.wind_speed_set = reader.read_float()
        self.cloud_number = reader.read_byte()
        self.trees = [reader.read_int() for _ in range(3)]
        self.tree_styles = [reader.read_byte() for _ in range(4)]
        self.cave_backs = [reader.read_int() for _ in range(3)]
        self.cave_back_styles = [reader.read_byte() for _ in range(4)]
        self.forest_tree_top_styles = [reader.read_byte() for _ in range(4)]
        self.corruption_tree_top_style = reader.read_byte()
        self.jungle_tree_top_style = reader.read_byte()
        self.snow_tree_top_style = reader.read_byte()
        self.hallow_tree_top_style = reader.read_byte()
        self.crimson_tree_top_style = reader.read_byte()
        self.desert_tree_top_style = reader.read_byte()
        self.ocean_tree_top_style = reader.read_byte()
        self.glowing_mushroom_tree_top_style = reader.read_byte()
        self.underworld_tree_top_style = reader.read_byte()
        self.rain = reader.read_float()
        self.event_info = reader.read_ulong()
        self.ore_tiers_tiles = [reader.read_short() for _ in range(7)]
        self.invasion_type = reader.read_sbyte()
        self.lobby_id = reader.read_ulong()
        self.sandstorm_severity = reader.read_float()

    async def handle(self, world, player, evman):
        world.time = self.time
        world.day_info = self.day_info
        world.moon_phase = self.moon_phase
        world.max_tiles_x = self.max_tiles_x
        world.max_tiles_y = self.max_tiles_y
        world.spawn_x = self.spawn_x
        world.spawn_y = self.spawn_y
        world.world_surface = self.world_surface
        world.rock_layer = self.rock_layer
        world.world_id = self.world_id
        world.world_name = self.world_name
        world.game_mode = self.game_mode
        world.world_unique_id = self.world_unique_id
        world.world_generator_version = self.world_generator_version
        world.moon_type = self.moon_type
        world.tree_background = self.tree_background
        world.corruption_background = self.corruption_background
        world.jungle_background = self.jungle_background
        world.snow_background = self.snow_background
        world.hallow_background = self.hallow_background
        world.crimson_background = self.crimson_background
        world.desert_background = self.desert_background
        world.ocean_background = self.ocean_background
        world.unknown_background = self.unknown_background
        world.ice_back_style = self.ice_back_style
        world.jungle_back_style = self.jungle_back_style
        world.hell_back_style = self.hell_back_style
        world.wind_speed_set = self.wind_speed_set
        world.cloud_number = self.cloud_number
        world.trees = self.trees
        world.tree_styles = self.tree_styles
        world.cave_backs = self.cave_backs
        world.cave_back_styles = self.cave_back_styles
        world.forest_tree_top_styles = self.forest_tree_top_styles
        world.corruption_tree_top_style = self.corruption_tree_top_style
        world.jungle_tree_top_style = self.jungle_tree_top_style
        world.snow_tree_top_style = self.snow_tree_top_style
        world.hallow_tree_top_style = self.hallow_tree_top_style
        world.crimson_tree_top_style = self.crimson_tree_top_style
        world.desert_tree_top_style = self.desert_tree_top_style
        world.ocean_tree_top_style = self.ocean_tree_top_style
        world.glowing_mushroom_tree_top_style = self.glowing_mushroom_tree_top_style
        world.underworld_tree_top_style = self.underworld_tree_top_style
        world.rain = self.rain
        world.event_info = self.event_info
        world.ore_tiers_tiles = self.ore_tiers_tiles
        world.invasion_type = self.invasion_type
        world.lobby_id = self.lobby_id
        world.sandstorm_severity = self.sandstorm_severity



"""
AmmoID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto
from terrex.id.ItemID import ItemID


class AmmoID(IntEnum):
    NoneValue = 0
    Gel = 23
    Acorn = 27
    Arrow = 40
    Coin = 71
    FallenStar = 75
    Bullet = 97
    Sand = 169
    Dart = 283
    Rocket = 771
    Solution = 780
    Flare = 931
    Snowball = 949
    StyngerBolt = 1261
    CandyCorn = 1783
    JackOLantern = 1785
    Stake = 1836
    NailFriendly = 3108
    Count = 0


class AmmoIDSets:
    class Main:
        factory = SetFactory(ItemID.Count)
        IsArrow = factory.create_bool_set(False, AmmoID.Arrow, AmmoID.Stake)
        IsBullet = factory.create_bool_set(False, AmmoID.Bullet, AmmoID.CandyCorn)
        IsSpecialist = factory.create_bool_set(False, AmmoID.Rocket, AmmoID.StyngerBolt, AmmoID.JackOLantern, AmmoID.NailFriendly, AmmoID.Coin, AmmoID.Flare, AmmoID.Dart, AmmoID.Snowball, AmmoID.FallenStar, AmmoID.Gel)



from collections import deque
from threading import Lock
from typing import cast


class SetFactory:
    def __init__(self, size: int):
        if size == 0:
            raise ValueError("size cannot be 0")
        self._size = size
        self._queue_lock = Lock()
        self._int_buffer_cache: deque[list[int]] = deque()
        self._ushort_buffer_cache: deque[list[int]] = deque()
        self._bool_buffer_cache: deque[list[bool]] = deque()
        self._float_buffer_cache: deque[list[float]] = deque()

    def _get_buffer(self, cache: deque, dtype):
        with self._queue_lock:
            if cache:
                return cache.popleft()
            return [dtype()] * self._size

    def _recycle_buffer(self, buffer, dtype):
        with self._queue_lock:
            if dtype is bool:
                self._bool_buffer_cache.append(buffer)
            elif dtype is int:
                self._int_buffer_cache.append(buffer)
            elif dtype is float:
                self._float_buffer_cache.append(buffer)

    def create_bool_set(self, default_state: bool = False, *types: int) -> list[bool]:
        arr = cast(list[bool], self._get_buffer(self._bool_buffer_cache, bool))
        for i in range(self._size):
            arr[i] = default_state
        for t in types:
            if 0 <= t < self._size:
                arr[t] = not default_state
        return arr

    def create_int_set(self, default_state: int = -1, *inputs: int) -> list[int]:
        if len(inputs) % 2 != 0:
            raise ValueError("inputs length must be even")
        arr = cast(list[int], self._get_buffer(self._int_buffer_cache, int))
        for i in range(self._size):
            arr[i] = default_state
        for j in range(0, len(inputs), 2):
            idx = inputs[j]
            val = inputs[j + 1]
            if 0 <= idx < self._size:
                arr[idx] = val
        return arr

    def create_float_set(self, default_state: float, *inputs: float) -> list[float]:
        if len(inputs) % 2 != 0:
            raise ValueError("inputs length must be even")
        arr = cast(list[float], self._get_buffer(self._float_buffer_cache, float))
        for i in range(self._size):
            arr[i] = default_state
        for j in range(0, len(inputs), 2):
            idx = int(inputs[j])
            val = inputs[j + 1]
            if 0 <= idx < self._size:
                arr[idx] = val
        return arr

    def create_ushort_set(self, default_state: int, *inputs: int) -> list[int]:
        if len(inputs) % 2 != 0:
            raise ValueError("inputs length must be even")
        arr = cast(list[int], self._get_buffer(self._ushort_buffer_cache, int))
        for i in range(self._size):
            arr[i] = default_state
        for j in range(0, len(inputs), 2):
            idx = inputs[j]
            val = inputs[j + 1]
            if 0 <= idx < self._size:
                arr[idx] = val
        return arr

    def create_custom_set(self, default_state, *inputs) -> list:
        if len(inputs) % 2 != 0:
            raise ValueError("inputs length must be even")
        arr = [default_state] * self._size
        for j in range(0, len(inputs), 2):
            key = inputs[j]
            val = inputs[j + 1]
            idx = int(key)
            if 0 <= idx < self._size:
                arr[idx] = val
        return arr

    def recycle(self, buffer, dtype=bool):
        if not buffer:
            return
        self._recycle_buffer(buffer, dtype)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import SyncPacket


class TeleportNPCThroughPortal(SyncPacket):
    id = MessageID.TeleportNPCThroughPortal

    def __init__(
        self,
        npc_id: int = 0,
        portal_color_index: int = 0,
        pos: Vec2 | None = None,
        vel: Vec2 | None = None,
    ):
        self.npc_id = npc_id
        self.portal_color_index = portal_color_index
        self.pos = pos or Vec2(0.0, 0.0)
        self.vel = vel or Vec2(0.0, 0.0)

    def read(self, reader: Reader):
        self.npc_id = reader.read_ushort()
        self.portal_color_index = reader.read_ushort()
        self.pos = Vec2.read(reader)
        self.vel = Vec2.read(reader)

    def write(self, writer: Writer):
        writer.write_ushort(self.npc_id)
        writer.write_ushort(self.portal_color_index)
        self.pos.write(writer)
        self.vel.write(writer)



from terrex.id import MessageID
from terrex.localization.network_text import NetworkText
from terrex.net.streamer import Reader
from terrex.net.structure.rgb import Rgb
from terrex.packet.base import ServerPacket


class SmartTextMessage(ServerPacket):
    id = MessageID.SmartTextMessage

    def __init__(
        self, color: Rgb | None = None, message: NetworkText | None = None, message_length: int = 0
    ):
        self.color = color
        self.message = message or NetworkText()
        self.message_length = message_length

    def read(self, reader: Reader) -> None:
        self.color = Rgb.read(reader)
        self.message = NetworkText.read(reader)
        self.message_length = reader.read_short()



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class TileFrameSection(ServerPacket):
    id = MessageID.TileFrameSection

    def __init__(self, start_x: int = 0, start_y: int = 0, end_x: int = 0, end_y: int = 0):
        self.start_x = start_x
        self.start_y = start_y
        self.end_x = end_x
        self.end_y = end_y

    def read(self, reader: Reader):
        self.start_x = reader.read_short()
        self.start_y = reader.read_short()
        self.end_x = reader.read_short()
        self.end_y = reader.read_short()



"""
DustID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class DustID(IntEnum):
    Dirt = 0
    Stone = 1
    Grass = 2
    GrassBlades = 3
    TintableDust = 4
    t_Slime = 4
    Blood = 5
    t_Flesh = 5
    Torch = 6
    Iron = 8
    t_SteampunkMetal = 8
    Copper = 9
    Gold = 10
    t_Golden = 10
    Silver = 11
    LifeCrystal = 12
    Glass = 13
    Corruption = 14
    JesterSparkleBlue = 15
    Cloud = 16
    CorruptPlants = 17
    CorruptGibs = 18
    t_Meteor = 23
    Bone = 26
    Shadowflame = 27
    WaterCandle = 29
    Web = 30
    Smoke = 31
    Sand = 32
    UnderwaterBubble = 34
    Lava = 35
    Ash = 36
    JunglePlants = 40
    t_Cactus = 40
    TintableDustLighted = 43
    HallowedPlants = 47
    JesterSparkleYellow = 57
    PinkStarfury = 58
    BlueTorch = 59
    RedTorch = 60
    GreenTorch = 61
    PurpleTorch = 62
    WhiteTorch = 63
    YellowTorch = 64
    DemonTorch = 65
    RainbowTorch = 66
    BlueCrystalShard = 68
    t_Crystal = 68
    PinkCrystalShard = 69
    PurpleCrystalShard = 70
    CursedTorch = 75
    t_LivingWood = 78
    t_PearlWood = 78
    Ice = 80
    t_Frozen = 80
    Tin = 81
    Lead = 82
    Tungsten = 83
    Platinum = 84
    GemAmethyst = 86
    GemTopaz = 87
    GemSapphire = 88
    GemEmerald = 89
    GemRuby = 90
    GemDiamond = 91
    GreenMoss = 93
    BrownMoss = 94
    RedMoss = 95
    BlueMoss = 96
    PurpleMoss = 97
    Terra = 107
    PureSpray = 110
    HallowSpray = 111
    CorruptSpray = 112
    MushroomSpray = 113
    CrimsonSpray = 114
    Crimson = 115
    CrimsonPlants = 125
    Flare = 127
    IceTorch = 135
    GemAmber = 138
    Confetti = 139
    TintablePaint = 143
    t_Honey = 147
    t_Lihzahrd = 148
    Honey = 152
    UltraBrightTorch = 156
    Chlorophyte = 157
    OrangeTorch = 158
    BlueGreenElectricity = 160
    HeatRay = 162
    PlanteraPetal = 166
    PlanteraGrass = 167
    PlanteraBulb = 168
    IchorTorch = 169
    IchorSplash = 170
    Shadowbeam = 173
    Eater = 184
    BlueFlare = 187
    SnowflakeIce = 197
    t_BorealWood = 214
    Fireworks = 219
    Electric = 226
    t_Martian = 226
    MartianBlood = 227
    GoldFlame = 228
    Vortex = 229
    BoneTorch = 234
    Marble = 236
    t_Marble = 236
    Mothron = 237
    MothronEgg = 238
    Silk = 239
    Granite = 240
    t_Granite = 240
    MarblePot = 241
    PinkTorch = 242
    PinkSlime = 243
    CopperCoin = 244
    SilverCoin = 245
    GoldCoin = 246
    PlatinumCoin = 247
    EnchantedNightcrawler = 248
    Grubby = 249
    Sluggy = 250
    Buggy = 251
    CrystalPulse = 254
    CrystalPulse2 = 255
    ToxicBubble = 256
    BubbleBlock = 257
    LavaMoss = 258
    SolarFlare = 259
    SparksMech = 260
    AncientLight = 261
    AmberBolt = 262
    PortalBolt = 263
    PortalBoltTrail = 264
    LunarOre = 265
    SomethingRed = 266
    RainbowMk2 = 267
    Sandstorm = 268
    Sandnado = 269
    FlameBurst = 270
    HealingPlus = 271
    WitherLightning = 272
    GreenBlood = 273
    ApprenticeStorm = 274
    DungeonBlue = 275
    DungeonGreen = 276
    DungeonPink = 277
    FireworksRGB = 278
    SilverFlame = 279
    BeachShell = 280
    RedStarfish = 281
    PinkStarfish = 282
    YellowStarfish = 283
    FoodPiece = 284
    GolfPaticle = 285
    OrangeStainedGlass = 286
    DesertPot = 287
    DesertWater2 = 288
    SeaOatsOasis = 289
    SeaOatsBeach = 290
    OasisCactus = 291
    YellowStarDust = 292
    DesertTorch = 293
    CoralTorch = 294
    CorruptTorch = 295
    CrimsonTorch = 296
    HallowedTorch = 297
    JungleTorch = 298
    KryptonMoss = 299
    XenonMoss = 300
    ArgonMoss = 301
    Terragrim = 302
    SteampunkSteam = 303
    Firefly = 304
    VioletMoss = 305
    SparkForLightDisc = 306
    MushroomTorch = 307
    ShimmerSplash = 308
    ShimmerSpark = 309
    ShimmerTorch = 310
    SandSpray = 311
    SnowSpray = 312
    DirtSpray = 313
    LunarRust = 314
    DarkCelestial = 315
    Astra = 316
    CosmicEmber = 317
    Cryocore = 318
    Mercury = 319
    StarRoyale = 320
    Heavenforge = 321
    Poop = 322
    MoonBoulder = 323
    ShimmerStream = 324
    PinkPhaseblade = 325
    LavaCloud = 326
    StarCloud = 327
    RainbowCloud = 328
    BirdDroppings = 329
    PrettyMirror = 330




import asyncio
from collections.abc import Awaitable, Callable

from .dispatcher import Dispatcher
from .filter.base import EventFilter
from .types import BaseEvent


class EventManager:
    def __init__(self, dispatcher: Dispatcher):
        self._dispatcher = dispatcher

    def on_event(self, filter: EventFilter):
        def decorator(
            func: Callable[[BaseEvent], Awaitable[None]],
        ) -> Callable[[BaseEvent], Awaitable[None]]:
            self._dispatcher.register(filter, func)
            return func

        return decorator

    def raise_event(self, event: BaseEvent):
        loop = asyncio.get_running_loop()
        loop.create_task(self._dispatcher.dispatch(event))

    def stop(self):
        self._dispatcher.shutdown()



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class PlayerLifeMana(SyncPacket):
    id = MessageID.PlayerLifeMana

    def __init__(self, player_id: int = 0, hp: int = 100, max_hp: int = 100):
        self.player_id = player_id
        self.hp = hp
        self.max_hp = max_hp

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.hp = reader.read_ushort()
        self.max_hp = reader.read_ushort()

    def write(self, writer: Writer):
        writer.write_byte(self.player_id)
        writer.write_ushort(self.hp)
        writer.write_ushort(self.max_hp)



"""
MusicID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 07:02 UTC
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class MusicID(IntEnum):
    NoneValue = 0
    OverworldDay = 1
    Eerie = 2
    OverworldNight = 3
    Underground = 4
    Boss1 = 5
    TitleClassic = 6
    JungleDay = 7
    Corruption = 8
    Hallow = 9
    UndergroundCorruption = 10
    UndergroundHallow = 11
    Boss2 = 12
    Boss3 = 13
    Snow = 14
    SpaceNight = 15
    Crimson = 16
    Golem = 17
    OverworldDayAlt = 18
    Rain = 19
    Ice = 20
    Desert = 21
    OceanDay = 22
    Dungeon = 23
    Plantera = 24
    Boss5 = 25
    Temple = 26
    Eclipse = 27
    RainAmbience = 28
    Mushroom = 29
    PumpkinMoon = 30
    UndergroundAlt = 31
    FrostMoon = 32
    UndergroundCrimson = 33
    LunarPillars = 34
    PirateInvasion = 35
    Underworld = 36
    MartianMadness = 37
    MoonLord = 38
    GoblinArmy = 39
    Sandstorm = 40
    OldOnesArmy = 41
    SpaceDay = 42
    OceanNight = 43
    WindyDay = 44
    WindAmbience = 45
    TownDay = 46
    TownNight = 47
    SlimeRain = 48
    OverworldDayRemix = 49
    TitleJourneysBeginningWithIntro = 50
    TitleJourneysBeginning = 51
    Storm = 52
    Graveyard = 53
    UndergroundJungle = 54
    JungleNight = 55
    QueenSlime = 56
    EmpressOfLight = 57
    DukeFishron = 58
    MorningRain = 59
    TitleAlt = 60
    UndergroundDesert = 61
    OtherworldRain = 62
    OtherworlddDay = 63
    OtherworldNight = 64
    OtherworldUnderground = 65
    OtherworldDesert = 66
    OtherworldOcean = 67
    OtherworldMushroom = 68
    OtherworldDungeon = 69
    OtherworldSpace = 70
    OtherworldUnderworld = 71
    OtherworldSnow = 72
    OtherworldCorruption = 73
    OtherworldUndergroundCorruption = 74
    OtherworldCrimson = 75
    OtherworldUndergroundCrimson = 76
    OtherworldIce = 77
    OtherworldUndergroundHallow = 78
    OtherworldEerie = 79
    OtherworldBoss2 = 80
    OtherworldBoss1 = 81
    OtherworldInvasion = 82
    OtherworldLunarPillars = 83
    OtherworldMoonLord = 84
    OtherworldPlantera = 85
    OtherworldJungle = 86
    OtherworldWallOfFlesh = 87
    OtherworldHallow = 88
    JourneysEnd = 89
    Deerclops = 90
    Aether = 91
    Destroyer = 92
    KingSlime = 93
    LunaticCultist = 94
    QueenBeeAlt = 95
    QueenBee = 96
    Twins = 97
    SkeletronPrime = 98
    EaterOfWorlds = 99
    TorchGodVocals = 100
    TorchGodInstrumental = 101
    RainbowBoulderStart = 102
    RainbowBoulderLoop = 103
    Skeletron = 104
    Count = 105


class MusicIDSets:
    class Main:
        factory = SetFactory(MusicID.Count)
        CanBeRecorded = factory.create_bool_set(True, 0, 28, 45, 103, 102)



from dataclasses import dataclass

from terrex.net.creative_power import (
    CreativePower,
    creative_power_registry,
)
from terrex.net.streamer import Reader, Writer

from .net_module import NetSyncModule


@dataclass()
class NetCreativePowersModule(NetSyncModule):
    id: int = 5
    power: CreativePower | None = None

    @classmethod
    def create(cls, power: CreativePower) -> "NetCreativePowersModule":
        obj = cls()
        obj.power = power
        return obj

    def read(self, reader: Reader) -> None:
        power_id = reader.read_byte()
        power_cls = creative_power_registry.get(power_id)
        if power_cls is None:
            raise ValueError(f"Unknown CreativePower variant: {power_id}")
        if not issubclass(power_cls, CreativePower):
            raise ValueError(f"Registry entry for {power_id} is not a subclass of CreativePower")
        self.power = power_cls()
        self.power.read(reader)

    def write(self, writer: Writer) -> None:
        if self.power is None:
            raise ValueError("power must not be None")
        writer.write_byte(self.power.id)
        self.power.write(writer)



"""
ProjectileDrawLayerID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class ProjectileDrawLayerID(IntEnum):
    NoneValue = 0
    BehindNPCsAndTiles = 1
    BehindNPCs = 2
    BehindProjectiles = 3
    OverPlayers = 4
    OverWiresUI = 5
    FirstFractals = 6
    HeldProj = 7
    HeldProjOverHand = 8




from dataclasses import is_dataclass
from enum import Enum
from typing import Any

from terrex.localization.network_text import NetworkText
from terrex.net.bits_byte import BitsByte


def stringify_network_text(
    value: Any, depth: int = 0, max_depth: int = 20, seen: set[Any] | None = None
) -> Any:
    if seen is None:
        seen = set()

    from terrex.localization.localization import get_translation
    from terrex.localization.network_text import NetworkText

    if not isinstance(value, NetworkText):
        return repr(value)

    if depth > max_depth:
        return {
            "mode": "<truncated>",
            "key": "<truncated>",
            "translated": "<truncated>",
            "substitutions": [],
        }

    obj_id = id(value)
    seen.add(obj_id)
    try:
        return {
            "mode": value.mode.name,
            "key": value.text,
            "translated": get_translation(value),
            "substitutions": [
                stringify_value(sub, depth + 1, max_depth, seen) for sub in value.substitutions
            ],
        }
    finally:
        seen.remove(obj_id)


def stringify_value(
    value: Any, depth: int = 0, max_depth: int = 20, seen: set[Any] | None = None
) -> Any:
    """
    Recursive stringify for logging/repr: NetworkText -> dict, bytes -> hex, list/dict/dataclass recursive.
    Shared for Packet/NetModule. Supports cycle detection and max depth truncation.
    """
    if seen is None:
        seen = set()
    obj_id = id(value)
    if obj_id in seen:
        return "<cycle>"
    if depth > max_depth:
        return f"<truncated at depth {depth}>"
    seen.add(obj_id)
    try:
        if isinstance(value, NetworkText):
            return stringify_network_text(value, depth, max_depth, seen)
        if isinstance(value, Enum):
            return f"{value.name}({value.value})"
        if isinstance(value, dict):
            return {
                str(key): stringify_value(item, depth + 1, max_depth, seen)
                for key, item in value.items()
            }
        if isinstance(value, BitsByte):
            return str(value)
        if isinstance(value, (bytes, bytearray)):
            return value.hex()
        if isinstance(value, list):
            return [stringify_value(item, depth + 1, max_depth, seen) for item in value]
        if is_dataclass(value) or hasattr(value, "__dict__"):
            return {
                str(key): stringify_value(getattr(value, key), depth + 1, max_depth, seen)
                for key in vars(value).keys()
            }
        return repr(value)
    finally:
        seen.remove(obj_id)



from terrex.net.bits_byte import BitsByte


class WorldZone:
    def __init__(
        self,
        zone1: BitsByte | None = None,
        zone2: BitsByte | None = None,
        zone3: BitsByte | None = None,
        zone4: BitsByte | None = None,
        zone5: BitsByte | None = None,
    ):
        self.zone1 = zone1 or BitsByte()
        self.zone2 = zone2 or BitsByte()
        self.zone3 = zone3 or BitsByte()
        self.zone4 = zone4 or BitsByte()
        self.zone5 = zone5 or BitsByte()

    def update(
        self,
        zone1: BitsByte | None = None,
        zone2: BitsByte | None = None,
        zone3: BitsByte | None = None,
        zone4: BitsByte | None = None,
        zone5: BitsByte | None = None,
    ) -> None:
        if zone1:
            self.zone1 = zone1
        if zone2:
            self.zone2 = zone2
        if zone3:
            self.zone3 = zone3
        if zone4:
            self.zone4 = zone4
        if zone5:
            self.zone5 = zone5

    # zone1
    @property
    def dungeon(self):
        return self.zone1[0]

    @dungeon.setter
    def dungeon(self, val):
        self.zone1[0] = val

    @property
    def corrupt(self):
        return self.zone1[1]

    @corrupt.setter
    def corrupt(self, val):
        self.zone1[1] = val

    @property
    def hallow(self):
        return self.zone1[2]

    @hallow.setter
    def hallow(self, val):
        self.zone1[2] = val

    @property
    def meteor(self):
        return self.zone1[3]

    @meteor.setter
    def meteor(self, val):
        self.zone1[3] = val

    @property
    def jungle(self):
        return self.zone1[4]

    @jungle.setter
    def jungle(self, val):
        self.zone1[4] = val

    @property
    def snow(self):
        return self.zone1[5]

    @snow.setter
    def snow(self, val):
        self.zone1[5] = val

    @property
    def crimson(self):
        return self.zone1[6]

    @crimson.setter
    def crimson(self, val):
        self.zone1[6] = val

    @property
    def water_candle(self):
        return self.zone1[7]

    @water_candle.setter
    def water_candle(self, val):
        self.zone1[7] = val

    # zone2
    @property
    def peace_candle(self):
        return self.zone2[0]

    @peace_candle.setter
    def peace_candle(self, val):
        self.zone2[0] = val

    @property
    def tower_solar(self):
        return self.zone2[1]

    @tower_solar.setter
    def tower_solar(self, val):
        self.zone2[1] = val

    @property
    def tower_vortex(self):
        return self.zone2[2]

    @tower_vortex.setter
    def tower_vortex(self, val):
        self.zone2[2] = val

    @property
    def tower_nebula(self):
        return self.zone2[3]

    @tower_nebula.setter
    def tower_nebula(self, val):
        self.zone2[3] = val

    @property
    def tower_stardust(self):
        return self.zone2[4]

    @tower_stardust.setter
    def tower_stardust(self, val):
        self.zone2[4] = val

    @property
    def desert(self):
        return self.zone2[5]

    @desert.setter
    def desert(self, val):
        self.zone2[5] = val

    @property
    def glowshroom(self):
        return self.zone2[6]

    @glowshroom.setter
    def glowshroom(self, val):
        self.zone2[6] = val

    @property
    def underground_desert(self):
        return self.zone2[7]

    @underground_desert.setter
    def underground_desert(self, val):
        self.zone2[7] = val

    # zone3
    @property
    def sky_height(self):
        return self.zone3[0]

    @sky_height.setter
    def sky_height(self, val):
        self.zone3[0] = val

    @property
    def overworld_height(self):
        return self.zone3[1]

    @overworld_height.setter
    def overworld_height(self, val):
        self.zone3[1] = val

    @property
    def dirt_layer_height(self):
        return self.zone3[2]

    @dirt_layer_height.setter
    def dirt_layer_height(self, val):
        self.zone3[2] = val

    @property
    def rock_layer_height(self):
        return self.zone3[3]

    @rock_layer_height.setter
    def rock_layer_height(self, val):
        self.zone3[3] = val

    @property
    def underworld_height(self):
        return self.zone3[4]

    @underworld_height.setter
    def underworld_height(self, val):
        self.zone3[4] = val

    @property
    def beach(self):
        return self.zone3[5]

    @beach.setter
    def beach(self, val):
        self.zone3[5] = val

    @property
    def rain(self):
        return self.zone3[6]

    @rain.setter
    def rain(self, val):
        self.zone3[6] = val

    @property
    def sandstorm(self):
        return self.zone3[7]

    @sandstorm.setter
    def sandstorm(self, val):
        self.zone3[7] = val

    # zone4
    @property
    def old_one_army(self):
        return self.zone4[0]

    @old_one_army.setter
    def old_one_army(self, val):
        self.zone4[0] = val

    @property
    def granite(self):
        return self.zone4[1]

    @granite.setter
    def granite(self, val):
        self.zone4[1] = val

    @property
    def marble(self):
        return self.zone4[2]

    @marble.setter
    def marble(self, val):
        self.zone4[2] = val

    @property
    def hive(self):
        return self.zone4[3]

    @hive.setter
    def hive(self, val):
        self.zone4[3] = val

    @property
    def gem_cave(self):
        return self.zone4[4]

    @gem_cave.setter
    def gem_cave(self, val):
        self.zone4[4] = val

    @property
    def lihzhard_temple(self):
        return self.zone4[5]

    @lihzhard_temple.setter
    def lihzhard_temple(self, val):
        self.zone4[5] = val

    @property
    def graveyard(self):
        return self.zone4[6]

    @graveyard.setter
    def graveyard(self, val):
        self.zone4[6] = val

    @property
    def shadow_candle(self):
        return self.zone4[7]

    @shadow_candle.setter
    def shadow_candle(self, val):
        self.zone4[7] = val

    # zone5
    @property
    def shimmer(self):
        return self.zone5[0]

    @shimmer.setter
    def shimmer(self, val):
        self.zone5[0] = val



"""
PlayerVariantID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class PlayerVariantID(IntEnum):
    MaleStarter = 0
    MaleSticker = 1
    MaleGangster = 2
    MaleCoat = 3
    FemaleStarter = 4
    FemaleSticker = 5
    FemaleGangster = 6
    FemaleCoat = 7
    MaleDress = 8
    FemaleDress = 9
    MaleDisplayDoll = 10
    FemaleDisplayDoll = 11
    Count = 12


class PlayerVariantIDSets:
    class Main:
        factory = SetFactory(PlayerVariantID.Count)
        Male = factory.create_bool_set(False, 0, 1, 2, 3, 8, 10)
        AltGenderReference = factory.create_int_set(0, 0, 4, 4, 0, 1, 5, 5, 1, 2, 6, 6, 2, 3, 7, 7, 3, 8, 9, 9, 8, 10, 11, 11, 10)
        VariantOrderMale = [0, 1, 2, 3, 8, 10]
        VariantOrderFemale = [4, 5, 6, 7, 9, 11]



from dataclasses import dataclass

from terrex.id import LiquidID
from terrex.net.streamer import Reader, Writer


@dataclass
class Liquid:
    x: int
    y: int
    amount: int
    type: LiquidID

    @classmethod
    def read(cls, reader: Reader) -> 'Liquid':
        pos = reader.read_int()
        amount = reader.read_byte()
        type = LiquidID(reader.read_byte())
        x = (pos >> 16) & 0xFFFF
        y = pos & 0xFFFF
        return cls(x, y, amount, type)

    def write(self, writer: Writer) -> None:
        pos = (self.x << 16) | self.y
        writer.write_int(pos)
        writer.write_byte(self.amount)
        writer.write_byte(self.type)



from .chat_command import ChatCommand
from .difficulty_level import GameDifficultyLevel
from .mode import NetMode
from .section_size import SectionSize
from .teleport_pylon_operation import TeleportPylonOperation
from .teleport_pylon_type import TeleportPylonType
from .teleport_request_type import TeleportRequestType
from .teleport_type import TeleportType
from .world_size import WorldSize

__all__ = [
    'ChatCommand',
    'GameDifficultyLevel',
    'NetMode',
    'SectionSize',
    'TeleportPylonType',
    'TeleportPylonOperation',
    'TeleportRequestType',
    'TeleportType',
    'WorldSize',
]



from terrex.event.types import BlockedEvent
from terrex.id import MessageID
from terrex.localization.network_text import NetworkText
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class Kick(ServerPacket):
    id = MessageID.Kick

    def __init__(self, reason: NetworkText | None = None):
        self.reason = reason or NetworkText()

    def read(self, reader: Reader):
        self.reason = NetworkText.read(reader)

    async def handle(self, world, player, evman):
        evman.raise_event(BlockedEvent(self, self.reason.text))



from .any_var import AnyVar


class StrVar(AnyVar):
    def __init__(self, default: str, *, max_len: int | None = None, **kw):
        self._max_len = max_len
        super().__init__(default, **kw)

    def _validate(self, value):
        if not isinstance(value, str):
            raise TypeError(f"'{self.get_field_name()}' expects str")
        if self._max_len is not None and len(value) > self._max_len:
            raise ValueError(f"'{self.get_field_name()}' exceeds max length {self._max_len}")



from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer

from .net_module import NetClientModule


@dataclass()
class NetCreativeUnlocksPlayerReportModule(NetClientModule):
    id: int = 6
    player_id: int | None = None
    item_id: int | None = None
    amount: int = 0

    @classmethod
    def create(
        cls, player_id: int, item_id: int, amount: int = 0
    ) -> "NetCreativeUnlocksPlayerReportModule":
        obj = cls()
        obj.player_id = player_id
        obj.item_id = item_id
        obj.amount = amount
        return obj

    def read(self, reader: Reader) -> None:
        self.player_id = reader.read_byte()
        self.item_id = reader.read_ushort()
        self.amount = reader.read_ushort()

    def write(self, writer: Writer) -> None:
        if self.player_id is None or self.item_id is None:
            raise ValueError("player_id and item_id must not be None")
        writer.write_byte(self.player_id)
        writer.write_ushort(self.item_id)
        writer.write_ushort(self.amount)



import json
from enum import Enum
from pathlib import Path

from terrex.localization.network_text import NetworkText, NetworkTextMode


class LocaleType(Enum):
    LEGACY = "Legacy"
    GAME = "Game"
    ITEMS = "Items"
    NPCS = "NPCs"
    PROJECTILES = "Projectiles"
    TOWN = "Town"


def get_translation(
    net_text: NetworkText, lang: str = "en-US", locale_type: LocaleType = LocaleType.LEGACY
) -> str:
    """
    Translates NetString.text as a localization key with substitutions support.
    Example: "LegacyMultiplayer.1" -> "Incorrect password"
    Supports LITERAL, FORMATTABLE, LOCALIZATION_KEY.
    """
    if net_text.mode == NetworkTextMode.LITERAL:
        return net_text.text

    base_text = net_text.text

    if net_text.mode == NetworkTextMode.LOCALIZATION_KEY:
        if "." not in base_text:
            pass  # fallback to base_text
        else:
            parts = base_text.rsplit(".", 1)
            category, subkey = parts
            path = Path(f"locale/{lang}/{lang}.{locale_type.value}.json")
            if path.exists():
                try:
                    with open(path, encoding="utf-8") as f:
                        data = json.load(f)
                    if category in data and subkey in data[category]:
                        base_text = data[category][subkey]
                except (json.JSONDecodeError, KeyError, ValueError):
                    pass

    subs_trans = [get_translation(sub, lang, locale_type) for sub in net_text.substitutions]
    try:
        return base_text.format(*subs_trans)
    except (ValueError, KeyError):
        return base_text



import zlib

from terrex.event.types import WorldSectionUpdateEvent
from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class TileSection(ServerPacket):
    id = MessageID.TileSection

    def read(self, reader: Reader) -> None:
        compressed_data = reader.remaining()
        decompressed = zlib.decompress(compressed_data, -zlib.MAX_WBITS)
        section_reader = Reader(decompressed)

        self.x_start = section_reader.read_int()
        self.y_start = section_reader.read_int()
        self.width = section_reader.read_short()
        self.height = section_reader.read_short()

        self.section_reader = section_reader

    async def handle(self, world, player, evman):
        if self.width < 0 or self.width > 200 or self.height < 0 or self.height > 150:
            return

        if not self.section_reader:
            return

        from terrex.net.structure.world_section import WorldSection

        section = WorldSection(self.x_start, self.y_start, self.width, self.height)
        section.read(self.section_reader, world)

        print(f"Loaded {section}")

        evman.raise_event(WorldSectionUpdateEvent(self, section))
